package edu.sdsc.globusproxy;

import java.security.GeneralSecurityException;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.EOFException;
import java.io.File;

import org.globus.common.CoGProperties;
import org.globus.common.Version;
import org.globus.gsi.OpenSSLKey;
import org.globus.gsi.CertUtil;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.GSIConstants;
import org.globus.gsi.TrustedCertificates;
import org.globus.gsi.bc.BouncyCastleOpenSSLKey;
import org.globus.gsi.bc.BouncyCastleCertProcessingFactory;
import org.globus.gsi.proxy.ext.ProxyPolicy;
import org.globus.gsi.proxy.ext.ProxyCertInfo;
import org.globus.gsi.proxy.ProxyPolicyHandler;
import org.globus.gsi.proxy.ProxyPathValidator;
import org.globus.gsi.proxy.ProxyPathValidatorException;
import org.globus.util.Util;

import org.globus.tools.ProxyInit;

public class  ProxyInitTool extends ProxyInit {

    private PrivateKey userKey = null;

    public void init(String [] args) {
        verify(args[1], "User key");
        verify(args[0], "User certificate");
    }

    private static void verify(String file, String msg) {
        if (file == null) error(msg + " not specified.");

        File f = new File(file);
        if (!f.exists() || f.isDirectory()) error(msg + " not found.");
    }


   public String getProxyString(String certFile, String keyFile, String passphrase) {

        int bits         = 512;
        int lifetime     = 3600 * 12;
        boolean debug    = false;
        boolean quiet    = false;
        boolean verify   = false;
        boolean pkcs11   = false;
        boolean limited  = false;
        int pathLen      = -1;
        int proxyType    = -1;
        String policyLanguage = null;
        String policyFile = null;

        CoGProperties properties = CoGProperties.getDefault();

        boolean useGsi3     = true;
        boolean globusStyle = false;
        String proxyFile    = properties.getProxyFile();
        //String keyFile      = "/Users/mock/.globus/userkey.pem";
        //String certFile     = "/Users/mock/.globus/usercert.pem";

	ProxyInitTool init = new ProxyInitTool();


        CertUtil.init();
                
        ProxyCertInfo proxyCertInfo = null;

        init.setBits(bits);
        init.setLifetime(lifetime);

	//xxxx fix
	proxyType = GSIConstants.GSI_2_PROXY;

        init.setProxyType(proxyType);
        init.setProxyCertInfo(proxyCertInfo);

        init.setDebug(debug);
        init.setQuiet(quiet);

        return init.createProxy(certFile,
                         keyFile,
                         passphrase,
                         verify,
                         globusStyle,
                         proxyFile);
   }

   public String createProxy (String cert,
                            String key,
                            String passphrase,
                            boolean verify,
                            boolean globusStyle,
                            String proxyFile) {

	String result = null;
        init(new String [] {cert, key});
        loadCertificate(cert);
	loadKey(key, passphrase);
	sign();


        OutputStream out = null;
        try {

            ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
            proxy.save(byteStream);
            result =  byteStream.toString();

        } catch (IOException e) {
            System.err.println("Failed to save proxy to a file: " +
                               e.getMessage());
            System.exit(-1);
        } 
/***
finally {
        }
****/
        dispose();
	return result;

   }

    public void loadCertificate(String arg) {
        try {
            certificate = CertUtil.loadCertificate(arg);
        } catch(IOException e) {
            System.err.println("Error: Failed to load cert: " + arg);
            System.exit(-1);
        } catch(GeneralSecurityException e) {
            System.err.println("Error: Unable to load user certificate: " +
                               e.getMessage());
            System.exit(-1);
        }
    }

    public void loadKey(String arg, String passphrase) {
        try {
            OpenSSLKey key = new BouncyCastleOpenSSLKey(arg);

            if (key.isEncrypted()) {
                String pwd = passphrase;

                if (pwd == null) {
                    System.exit(1);
                }

                key.decrypt(pwd);
            }

            userKey = key.getPrivateKey();

        } catch(IOException e) {
            System.err.println("Error: Failed to load key: " + arg);
            System.exit(-1);
        } catch(GeneralSecurityException e) {
            System.err.println("Error: Wrong pass phrase");
            if (debug) {
                e.printStackTrace();
            }
            System.exit(-1);
        }
    }

    public void loadKey(String arg) {
        try {
            OpenSSLKey key = new BouncyCastleOpenSSLKey(arg);

            if (key.isEncrypted()) {
                String pwd = null;

                if (quiet) {
                    pwd = Util.getInput("Enter GRID pass phrase: ");
                } else {
                    pwd = Util.getInput("Enter GRID pass phrase for this identity: ");
                }

                if (pwd == null) {
                    System.exit(1);
                }

                key.decrypt(pwd);
            }

            userKey = key.getPrivateKey();

        } catch(IOException e) {
            System.err.println("Error: Failed to load key: " + arg);
            System.exit(-1);
        } catch(GeneralSecurityException e) {
            System.err.println("Error: Wrong pass phrase");
            if (debug) {
                e.printStackTrace();
            }
            System.exit(-1);
        }
    }


    public void sign() {
        try {
            BouncyCastleCertProcessingFactory factory =
                BouncyCastleCertProcessingFactory.getDefault();

            proxy = factory.createCredential(new X509Certificate[] {certificate},
                                             userKey,
                                             bits,
                                             lifetime,
                                             proxyType,
                                             proxyCertInfo);
        } catch (GeneralSecurityException e) {
            System.err.println("Failed to create a proxy: " + e.getMessage());
            System.exit(-1);
        }
    }



}

