package edu.stanford.db.rdf.util;

import java.lang.ref.*;
import java.util.*;

public class Cache {

  Object[] clockArray;
  int clockPtr = 0;

  WeakHashMap map = new WeakHashMap();

  HashMap inverse = new HashMap(); // id -> object

  /* Reference queue for cleared WeakReferences */
  private ReferenceQueue queue = new ReferenceQueue();

  int discarded = 0;

  /* Remove all invalidated entries from the map, that is, remove all entries
     whose keys have been discarded.  This method should be invoked once by
     each public mutator in this class.  We don't invoke this method in
     public accessors because that can lead to surprising
     ConcurrentModificationExceptions. */
  private void processQueue() {
    WeakKey wk;
    while ((wk = (WeakKey)queue.poll()) != null) {
      discarded++;
      inverse.remove(wk);
//       LongIDAware n = (LongIDAware)wk.get();
//       if(n != null)
// 	removed += inverse.remove(new Long(n.getLongID()));
    }
  }


  public Cache(int cacheSize) {
    if(cacheSize == 0)
      cacheSize = 1; // to prevent unnecessary checks
    clockArray = new Object[cacheSize];
  }

  public Cache(int cacheSize, boolean hasInverse) {

    this(cacheSize);
    if(!hasInverse)
      inverse = null;
  }

  public Object get(Object key) {

    WeakReference r = (WeakReference)map.get(key);
//     if(map == cacheNamespaces && r != null)
//       System.out.println("---- weakref for " + key + " is " + r.get());
    return r != null ? r.get() : null;
  }

  public void put(Object key, Object value) {

    // make sure we cache the value at least for a while
    clockArray[clockPtr] = value;
    if(clockPtr + 1 >= clockArray.length)
      clockPtr = 0;
    else
      clockPtr++;
    
    if(inverse != null) {
      Object w = WeakKey.create((LongIDAware)value, queue);
      map.put(key, w);
      processQueue();
      inverse.put(w, w);
    } else {
      Object w = new WeakReference(value);
      map.put(key, w);
    }
  }

  public Object get(long id) {

    if(inverse != null) {
      processQueue();
      WeakKey r = (WeakKey)inverse.get(new WeakKey(id));
      return r != null ? r.get() : null;
    } else
      return null;
  }

  public String toString() {

    return "CACHE[" + map.size() + (inverse != null ? ", " + inverse.size() : "") + ", " + discarded + "]";
  }


    static private class WeakKey extends WeakReference {
	private long id;	/* Hashcode of key, stored here since the key
				   may be tossed by the GC */

      private WeakKey(long id) {
	super(null);
	this.id = id;
      }

	private WeakKey(LongIDAware k) {
	    super(k);
	    id = k.getLongID();
	}

	private static WeakKey create(LongIDAware k) {
	    if (k == null) return null;
	    else return new WeakKey(k);
	}

	private WeakKey(LongIDAware k, ReferenceQueue q) {
	    super(k, q);
	    id = k.getLongID();
	}

	private static WeakKey create(LongIDAware k, ReferenceQueue q) {
	    if (k == null) return null;
	    else return new WeakKey(k, q);
	}

        /* A WeakKey is equal to another WeakKey iff they both refer to objects
	   that are, in turn, equal according to their own equals methods */
	public boolean equals(Object o) {
	    if (this == o) return true;
	    if (!(o instanceof WeakKey)) return false;
	    return id == ((WeakKey)o).id;
	    //Object t = this.get();
	    //Object u = ((WeakKey)o).get();
	    //if ((t == null) || (u == null)) return false;
	    //if (t == u) return true;
	    //	    return t.equals(u);
	}

        public int hashCode() {
	    return (int)id;
	}

    }
}

