/**
 *    '$RCSfile: EcogridQueryDataCacheItem.java,v $'
 *
 *     '$Author: ruland $'
 *       '$Date: 2005/12/14 04:00:04 $'
 *   '$Revision: 1.14 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.datasource;

import java.io.File;
import java.io.StringReader;

import org.ecoinformatics.ecogrid.queryservice.util.EcogridQueryParser;
import org.ecoinformatics.ecogrid.queryservice.util.EcogridQueryTransformer;
import org.ecoinformatics.ecogrid.queryservice.util.EcogridResultsetParser;
import org.ecoinformatics.ecogrid.queryservice.util.EcogridResultsetTransformer;
import org.ecoinformatics.ecogrid.queryservice.query.QueryType;
import org.ecoinformatics.ecogrid.queryservice.resultset.ResultsetType;
import org.ecoinformatics.ecogrid.queryservice.resultset.ResultsetTypeRecord;
import org.ecoinformatics.ecogrid.queryservice.EcogridQueryClient;
import org.ecoinformatics.seek.querybuilder.DBUIUtils;
import org.kepler.objectmanager.cache.DataCacheManager;
import org.kepler.objectmanager.cache.DataCacheObject;
import org.w3c.dom.Node;

import ptolemy.util.StringUtilities;

/**
 * This class is not as generic as the name may indicate. It is designed to get the metadata to determine 
 * where the data is stored and then does a Ecogrid "get" to the data. 
 */
public class EcogridQueryDataCacheItem extends DataCacheObject
{
    private QueryType     _query     = null;
    private ResultsetType _resultset = null;
    
    /**
     * 
     *
     */
    public EcogridQueryDataCacheItem()
    {
        super();
    }
    
    /**
     * 
     */
    protected String getSerialClassName()
    {
      return EcogridQueryDataCacheItem.class.getName();
    }
    
    /* (non-Javadoc)
     * @see org.ecoinformatics.seek.datasource.DataCacheItem#doWork()
     */
    public void doWork()
    {
        //dbg.print("EcogridQueryDataCacheItem - doing Work mStatus "+mStatus, 2);
        
        if (mLocalFileName != null && mLocalFileName.length() > 0)
        {
            File cacheFile = new File(mLocalFileName);
            if (cacheFile.exists() && cacheFile.length() > 0)
            {
                mStatus = CACHE_OK;
            }
        }
        // this is for cancel search 
        try
        {
            Thread.sleep(10);
        }
        catch(InterruptedException e)
        {
           dbg.print("Interrupte after get local cache @@@@@@@@@@@", 2);
           return; 
        }
        
        if (!isReady())
        {
                  try
                  {
                       EcogridQueryClient ecogridClient = new
                                EcogridQueryClient(mResourceName);

                        _resultset = ecogridClient.query(_query);
                        // this is for cancel search 
                        try
                        {
                            Thread.sleep(10);
                        }
                        catch(InterruptedException e)
                        {
                           dbg.print("Interrupte after fatory search @@@@@@@@@@@", 2);
                           return; 
                        }
                    }                    
                    catch(Exception ee)
                    {
 
                       dbg.print("The exception in query is "+ee.getMessage(), 2);
                    }
                    if (_resultset != null)
                    {
                      ResultsetTypeRecord[] records = _resultset.getRecord();
                      if (records != null)
                      {
                          String rsStr = null;
                          try {
                              rsStr = EcogridResultsetTransformer.toXMLString(_resultset);
                          } 
                          catch (java.io.IOException ioe ) {
                            rsStr = new String();  
                          }
                        if (rsStr.length() > 0)
                        {
                            mData   = rsStr.getBytes();
                            mSize   = mData.length;
                            mStatus = CACHE_COMPLETE;
                            // this is for cancel search 
                            try
                            {
                                Thread.sleep(10);
                            }
                            catch(InterruptedException e)
                            {
                                dbg.print("Interrupted before save cache @@@@@@@@@@@", 2);
                               return; 
                            }
                            this.saveData();
                            DataCacheManager.getInstance().saveCache();
                        } else {
                            System.err.println("*** Resultset string was empty.");
                            mStatus = CACHE_COMPLETE;;
                        }
                    } else
                    {
                        System.err.println("*** Resultset records array was NULL!");
                        mStatus = CACHE_COMPLETE;;
                    }
                  } 
                  else
                  {
                    System.err.println("*** Resultset was NULL!");
                    mStatus = CACHE_COMPLETE;
                   }
                
         }        
    }
    
    /**
     * 
     * @param aNode
     */
    public boolean buildFromDOM(Node aNode)
    {
        if (!super.buildFromDOM(aNode))
        {
            return false;
        }

        if (aNode != null) 
        {
            String queryStr = DBUIUtils.findAttrValue(aNode, "query");
            if (queryStr != null && queryStr.length() > 0)
            {
                try
                {
                    StringReader       strReader = new StringReader(StringUtilities.unescapeForXML(queryStr));
                    EcogridQueryParser parser    = new EcogridQueryParser(strReader);
                    parser.parseXML();
                    _query = parser.getEcogridQuery();
                    
                } catch(Exception e)
                {
                    System.err.println("Error processing DOM ["+mName+"]["+mResourceName+"]");
                    System.err.println("queryStr [\n"+queryStr+"\n]");
                    
                    System.err.println(e);
                    e.printStackTrace();
                    return false;
                }
                
            }
        }
        return true;
    }
    
    /**
     * The string with the extra attrs (query)
     * @return
     */
    protected String getAttrsForXML()
    {
        String str = "";
        if (_query != null)
        {
            str = StringUtilities.escapeForXML(EcogridQueryTransformer.toXMLString(_query));
        }
        return " query=\"" + str +"\"\n";
    }
    
    /**
     * @param aQuery the UqeryType object
     */
    public void setQuery(QueryType aQuery)
    {
        _query = aQuery;
    }
    
    /**
     * Returns the data cached as a resultset object
     * @return
     */
    public ResultsetType getResultset()
    {
        if (isReady() && _resultset == null)
        {
            try {
            _resultset = EcogridResultsetParser.parseXMLFile(mLocalFileName);
            }
            catch( org.xml.sax.SAXException e ) {
                _resultset = null;
            }
        }
        return _resultset;
    }
}
