package org.ecoinformatics.seek.datasource.darwincore;

import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.NamedObj;
import ptolemy.vergil.icon.EditorIcon;
import ptolemy.vergil.kernel.attributes.ResizablePolygonAttribute;
import ptolemy.vergil.kernel.attributes.TextAttribute;

class DarwinCoreIcon {

    private static final String YELLOW = "{1.0, 1.0, 0.0, 1.0}";
    private static final String RED = "{1.0, 0.0, 0.0, 1.0}";
    private static final String BLACK = "{0.0, 0.0, 0.0, 1.0}";
    private static final String MAGENTA = "{1.0, 0.0, 1.0, 1.0}";

    private static final String TITLE_BINARY = "0101";
    private static final String TITLE_BUSY = "BUSY";
    private static final String TITLE_ERROR = "ERROR";

    /** Icon indicating the communication region. */
    private ResizablePolygonAttribute _poly1;

    private ResizablePolygonAttribute _poly2;

    private TextAttribute _text;

    /**
     * Create the DarwinCoreDataSource Icon.
     * 
     * @throws NameDuplicationException
     * @throws IllegalActionException
     */
    public DarwinCoreIcon(NamedObj parent) throws NameDuplicationException, IllegalActionException {
        EditorIcon node_icon = new EditorIcon(parent, "_icon");

        // Create the folder
        _poly1 = new ResizablePolygonAttribute(node_icon, "outer");
        // _poly1.vertices.setToken("{7,-27, 7,-8, 28,-10, 28,-27, 16,-27,
        // 15,-29, 7,-29}");
        _poly1.vertices.setToken("{0,0, 8,-2, 9,1, 24,-2, 24,20, 0,24, 0,0}");
        _poly1.width.setToken("24");
        _poly1.height.setToken("24");
        _poly1.centered.setToken("true");
        _poly1.lineColor.setToken(BLACK);

        _poly2 = new ResizablePolygonAttribute(node_icon, "inner");
        _poly2.vertices.setToken("{0,2, 24,-5, 24,20, 0,24, 0,2}");
        _poly2.width.setToken("24");
        _poly2.height.setToken("24");
        _poly2.centered.setToken("true");
        _poly2.lineColor.setToken(BLACK);

        _text = new TextAttribute(node_icon, "text");
        _text.textSize.setExpression("11");

        setIconStatus(TITLE_BINARY, YELLOW);

        node_icon.setPersistent(false);
    }

    /**
     * Set the icon to Busy state
     * 
     */
    public void setBusy() {
        setIconStatus(TITLE_BUSY, RED);
    }

    /**
     * Set the icon to Ready state
     * 
     */
    public void setReady() {
        setIconStatus(TITLE_BINARY, YELLOW);
    }

    /**
     * Set the icon to Error state
     * 
     */
    public void setError() {
        setIconStatus(TITLE_ERROR, MAGENTA);
    }

    /**
     * Set the text and color of the icon
     * 
     * @param aText
     * @param aColor
     */
    protected void setIconStatus(String aText, String aColor) {
        try {
            _text.text.setExpression(aText);
            _poly1.fillColor.setToken(aColor);
            _poly2.fillColor.setToken(aColor);
        } catch (Exception e) {
        }
    }

}
