/**
 *    '$RCSfile: Eml200DataSource.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2005/12/16 22:15:41 $'
 *   '$Revision: 1.74 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.datasource.eml.eml2;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.ecoinformatics.seek.dataquery.DBTableNameResolver;
import org.ecoinformatics.seek.dataquery.DBTablesGenerator;
import org.ecoinformatics.seek.dataquery.HsqlDataQueryAction;
import org.ecoinformatics.seek.datasource.EcogridCompressedDataCacheItem;
import org.ecoinformatics.seek.datasource.EcogridDataCacheItem;
import org.ecoinformatics.seek.datasource.EcogridGZippedDataCacheItem;
import org.ecoinformatics.seek.datasource.EcogridMetaDataCacheItem;
import org.ecoinformatics.seek.datasource.EcogridTarArchivedDataCacheItem;
import org.ecoinformatics.seek.datasource.EcogridZippedDataCacheItem;
import org.ecoinformatics.seek.ecogrid.EML2MetadataSpecification;
import org.ecoinformatics.seek.ecogrid.ResultRecord;
import org.ecoinformatics.seek.ecogrid.ResultTreeRoot;
import org.ecoinformatics.seek.querybuilder.DBQueryDef;
import org.ecoinformatics.seek.querybuilder.DBQueryDefParserEmitter;
import org.ecoinformatics.seek.querybuilder.DBSchemaParserEmitter;
import org.ecoinformatics.util.DelimitedReader;
import org.ecoinformatics.util.Util;
import org.kepler.objectmanager.cache.DataCacheListener;
import org.kepler.objectmanager.cache.DataCacheManager;
import org.kepler.objectmanager.cache.DataCacheObject;
import org.kepler.objectmanager.data.DataSourceControllerFactory;
import org.kepler.objectmanager.data.DataType;
import org.kepler.objectmanager.data.db.DSSchemaDef;
import org.kepler.objectmanager.data.db.DSSchemaIFace;
import org.kepler.objectmanager.data.db.DSTableFieldIFace;
import org.kepler.objectmanager.data.db.Entity;
import org.kepler.objectmanager.data.db.QBTableauFactory;
import org.kepler.objectmanager.data.text.TextComplexFormatDataReader;
import org.xml.sax.InputSource;

import ptolemy.actor.TypedIOPort;
import ptolemy.actor.gui.style.TextStyle;
import ptolemy.data.ArrayToken;
import ptolemy.data.BooleanToken;
import ptolemy.data.DoubleToken;
import ptolemy.data.IntToken;
import ptolemy.data.LongToken;
import ptolemy.data.RecordToken;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import ptolemy.data.UnsignedByteToken;
import ptolemy.data.expr.FileParameter;
import ptolemy.data.expr.StringParameter;
import ptolemy.data.type.ArrayType;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.RecordType;
import ptolemy.data.type.Type;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.NamedObj;
import ptolemy.kernel.util.Settable;
import ptolemy.kernel.util.StringAttribute;
import ptolemy.moml.MoMLChangeRequest;


/**
 * A ResultRecord that is in EML 2.0.0 format.
 */
public class Eml200DataSource extends ResultRecord implements DataCacheListener
{

    static final String DATATYPES[] = { DataType.INT, DataType.FLOAT,
            DataType.DOUBLE, DataType.LONG, DataType.STR};

    static final BaseType BASETYPES[] = { BaseType.INT, BaseType.DOUBLE,
            BaseType.DOUBLE, BaseType.LONG, BaseType.STRING};

    static private Hashtable _TypeHash = new Hashtable();

    private org.ecoinformatics.util.DebugPrinter dbg = 
                            org.ecoinformatics.util.DebugPrinter.getInstance();

    private QBTableauFactory _qbTableauFactory = null;

    private DataSourceControllerFactory _nodeController = null;
    
    ///////////////////////////////////////////////////////////////////
    //// private variables ////

    /**
     * Output indicator parameter.
     */
    private int _outputType = _NOPORTTYPE;
    
    private Vector _entityList  = new Vector();
    private Entity _selectedTableEntity = null;

    //private Vector[] _dataVectors = null;

    //private int _recordCount = 0;

    private Vector _columns = null;

    //private Vector _colTypes = new Vector();

    private DBQueryDef _queryDef = null;

    private DSSchemaIFace _schemaDef = null;
    private DSSchemaDef _schema       = new DSSchemaDef();

    private boolean _ignoreSchemaChange = false;

    private boolean _schemaAttrIsSet = false;

    private DBTablesGenerator _tableGenerator = null;
    
    //private Vector _cachedDataItemList = new Vector();

    private EcogridDataCacheItem _selectedCachedDataItem = null;

    private EcogridMetaDataCacheItem _cacheMetaDataItem = null;
    
    //private boolean _hasImageEntity = false;
    
    private InputStream _reader = null;
    
    private ResultSet _resultSet = null;
    
    private DelimitedReader _simpleDelimitedReader = null;
    
    private TextComplexFormatDataReader _complexFormatReader = null;
    
    private boolean _endOfStream = false;
    
    private int _numberOfBytesRead = 0;
    
    private byte[] _byteArray = new byte[200000];
    
    //private boolean _isZipDataFile = false;
    
    private String[] _targetFilePathInZip = null;
    
    private String _selectedDataCacheItemLocalFileName = null;
    
    private String _selectedDataCacheItemResourceName  = null;
    
    private int _numberOfEntities = 0;
    
    private int _numberOfDownloadEntities = 0;
    
    private int _numberOfFailedDownloadEntities = 0;
    
    private boolean _hasSQLCommand        = false;
    
    private Vector[] _columnVectorArray    = null;
    
    private String[] _selectedColumnLabelList = null;
    
    private Type[] _selectedColumnTypeList    = null;
    
    private RecordToken _recordToken          = null;
  
    // Constants used for more efficient execution.
    private static final int _NOPORTTYPE = 0;

    private static final int _ASTABLE = 1;

    private static final int _ASROW = 2;

    private static final int _ASFIELD = 3;
    
    private static final int _ASBYTEARRAy = 4;
    
    private static final int _ASUNZIPPEDFILENAME = 5;
    
    private static final int _ASCACHEFILENAME    = 6;
    
    private static final int _ASCOLUMNVECTOR     = 7;
    
    private static final int _ASCOLUMNBASEDRECORD = 8;
    
    private static final String _ASBYTEARRAYLABEL = "As Byte Array";
    
    private static final String _ASUNZIPPEDFILENAMELABEL = "As UnCompressed File Name";
    
    private static final String _ASCACHEFILENAMELABEL = "As Cache File Name";
    
    private static final String _ASCOLUMNVECTORLABEL  = "As Column Vector";
    
    private static final String _ASCOLUMNBASEDRECORDLABEL = "As ColumnBased Record";
    
    private static final String _BINARYDATAPORT   = "BinaryData";
    
    private static final String _ENDOFSTREAMPORT  = "EndOfStream";
    
    private static final String _UNZIPPEDFILENAMEPORT = "UnzippedFileName";
    
    private static final String _CACHELOCALFILENAMEPORT = "CacheLocalFileName";
    
    private static final String _CACHERESOURCENAMEPORT =  "CacheResourceName";
    
    private static final int   INDEXFORNOTFOUND        = -1;
    
    private static final int   DEFAULTINDEX            = 0;
    
    private static final String _SELECTEDENTITY        = "Selected Entity";
    
    private static final String RECORDPORTNAME         = "Record";
    
    private static final String ROWDATAPORTNAME        = "DataRow";

    ///////////////////////////////////////////////////////////////////
    //// ports and parameters ////

    public FileParameter _datasetPathAttr = null;

    public StringAttribute _sqlAttr = null;

    public StringAttribute _schemaAttr = null;

    public StringParameter _outputTypeAttr = null;
    
    public StringParameter _fileExtensionInZip = null;
    
    public StringParameter _selectedEntityAttr    = null;

    /**
     * Construct an actor with the given container and name.
     * 
     * @param container
     *            The container.
     * @param name
     *            The name of this actor.
     * @exception IllegalActionException
     *                If the actor cannot be contained by the proposed
     *                container.
     * @exception NameDuplicationException
     *                If the container already has an actor with this name.
     * @since
     */
    public Eml200DataSource(CompositeEntity container, String name)
            throws NameDuplicationException, IllegalActionException
    {
        super(container, name);
        _datasetPathAttr = new FileParameter(this, "EML File");

        _schemaAttr = new StringAttribute(this, "schemaDef");
        TextStyle schemaDefTS = new TextStyle(_schemaAttr, "schemaDef");

        _sqlAttr = new StringAttribute(this, "sqlDef");
        TextStyle sqlDefTS = new TextStyle(_sqlAttr, "sqlDef");
        _selectedEntityAttr = new StringParameter(this, _SELECTEDENTITY);
        _outputTypeAttr = new StringParameter(this, "outputType");
        _outputTypeAttr.setExpression("As Field");
        _outputTypeAttr.addChoice("As Field");
        _outputTypeAttr.addChoice("As Table");
        _outputTypeAttr.addChoice("As Row");
        _outputTypeAttr.addChoice(_ASBYTEARRAYLABEL);
        _outputTypeAttr.addChoice(_ASUNZIPPEDFILENAMELABEL);
        _outputTypeAttr.addChoice(_ASCACHEFILENAMELABEL);
        _outputTypeAttr.addChoice(_ASCOLUMNVECTORLABEL);
        _outputTypeAttr.addChoice(_ASCOLUMNBASEDRECORDLABEL);
        _outputType = _ASFIELD;
        
        _fileExtensionInZip = new StringParameter(this, "Target File Extension in Compressed File");

        // create tableau for editting the SQL String
        _qbTableauFactory = new QBTableauFactory(this, "_tableauFactory");
        
        // Create a node controller to control the context menu
        _nodeController = new DataSourceControllerFactory(this, 
                "_controllerFactory");

        if (_TypeHash.size() == 0) {
            for (int i = 0; i < DATATYPES.length; i++) {
                _TypeHash.put(DATATYPES[i], BASETYPES[i]);
            }
        }
    }

    /**
     * Initialize the actor prior to running in the workflow. This reads the
     * metadata and configures the ports.
     * 
     * @throws IllegalActionException
     */
    public void initialize() throws IllegalActionException
    {
       dbg.print("In initialize method", 2);
     
       if (_selectedTableEntity == null) 
       {
          throw new IllegalActionException("_selectedTableEnity is NULL!");
       }
       
       switch (_outputType) 
       {
         case _ASTABLE:
            initializeForFieldOrRowOrTable();
            break;

         case _ASROW:
            initializeForFieldOrRowOrTable();
            break;

          case _ASFIELD:
            initializeForFieldOrRowOrTable();
            break;
          case _ASBYTEARRAy:
            initializeAsByteArray();
            break;
          case _ASUNZIPPEDFILENAME:
            initializeAsUnCompressedFileName();
            break;
          case _ASCACHEFILENAME:
             initializeAsCacheFileName();
             break;
          case _ASCOLUMNVECTOR:
              initializeAsColumnVector();
              break;
          case _ASCOLUMNBASEDRECORD:
              initializeAsColumnBasedRecord();
              break;
          default:
            throw new IllegalActionException(this, "Unrecognized outputType: "
                    + _outputType);

        }
       
   
    }
    
    
    /*
     * Intialize for output as field, row or Table. It will generate query from 
     * sqlDef attribute(if sqlDef is empty, the generate query as select * from 
     * selectedEntity. Then run this query action and get resultset
     */
     private void initializeForFieldOrRowOrTable() throws IllegalActionException
     {
       String sqlStr = "";
       String sqlXMLStr = ( (Settable) _sqlAttr).getExpression();
       if (sqlXMLStr == null || sqlXMLStr.length() == 0) {
         sqlStr = "SELECT * FROM "
             + (_selectedTableEntity.getMappedName() != null ? _selectedTableEntity
                .getMappedName() : _selectedTableEntity.getName());
 
       }
       else {
         Hashtable mappedNameHash = new Hashtable();
         // should go through all enities
         int size = _entityList.size();
         for (int i=0; i< size; i++)
         {
             Entity entity = (Entity)_entityList.elementAt(i);
             if (entity.getMappedName() != null)
             {
                 mappedNameHash.put(entity.getName(), entity.getMappedName());
             }
         }
         DBQueryDef queryDef = DBQueryDefParserEmitter.parseQueryDef(
             _schemaDef, sqlXMLStr, mappedNameHash);
 
         sqlStr = DBQueryDefParserEmitter.createSQL(_schemaDef, queryDef);
       }
       
       dbg.print("The sql command is "+sqlStr, 2);
        // excuted query
       if (sqlStr != null && !sqlStr.trim().equals("")) {
         // if table gnerated successfully, we will run query
         if (_tableGenerator != null && _tableGenerator.getSuccessStatus()) {
           try {
             setIconStatus(TITLE_BUSY, RED);
             HsqlDataQueryAction queryAction = new HsqlDataQueryAction();
             queryAction.setSQL(sqlStr);
             queryAction.actionPerformed(null);
             _resultSet = queryAction.getResultSet();
           
             //_dataVectors = transformResultSet(result);
             
           }
           catch (Exception e) 
           {
             dbg.print("Error to run query is " + e.getMessage(), 2);
             throw new IllegalActionException(e.getMessage());
           }
 
         }
       }
       //if reustlset is null(this can be caused by db couldn't create table)
       // and we don't have any sql command
       //(this means we only have one data entity involve) or only has one entity at all
       // we would like to try read the selected entity data from datachache rather than
       // from db
          if (_resultSet == null &&  (!_hasSQLCommand || _numberOfEntities==1))
          {
              //System.out.println("in result set is null!!!!!!!!!!!!!!!!!!");
              _reader = _selectedCachedDataItem.getDataInputStream();
              try
              {
                 createDelimitedReader();
              }
              catch (Exception e) 
              {
                dbg.print("Error to run delimiter reader is  " + e.getMessage(), 2);
                throw new IllegalActionException(e.getMessage());
              }
              
          }
          setIconStatus(TITLE_BINARY, YELLOW);
       
       // need reset this value, otherwise next running can only get one value
       _endOfStream = false;
     }
     
     /*
      * Initialize as column vector for fire. This method will transfer a 
      * resultset into cloumn array or transfer a delimiter reader to column array
      */
     private void initializeAsColumnVector() throws IllegalActionException
     {
         initializeForFieldOrRowOrTable();
         Vector rowVector = new Vector();
         try
         {
             rowVector = gotRowVectorFromSource();
         }
         catch(Exception e)
         {
             throw new IllegalActionException(e.getMessage());
         }
         int size = rowVector.size();
         // initial the columnVector array
         _columnVectorArray = new Vector[size];
         for (int i=0; i<size; i++)
         {
             Vector column = new Vector();
             _columnVectorArray[i]= column;
             
         }
         
         while (!rowVector.isEmpty())
         {
             for (int i =0; i<size; i++)
             {
                 Vector column = _columnVectorArray[i];
                 String data = (String) rowVector.elementAt(i);
                 dbg.print("Add data "+ data +" into column vector " +i, 2);
                 column.add(data);
             }
             try
             {
                 rowVector = gotRowVectorFromSource();
             }
             catch(Exception e)
             {
                 throw new IllegalActionException(e.getMessage());
             }
            
         }
     }
     
     /*
      * Initialize as column based record for fire. 
      */
     private void initializeAsColumnBasedRecord() throws IllegalActionException
     {
         //get cloumnDataArray 
         initializeAsColumnVector();
         //generate record
         int size = _selectedColumnLabelList.length;
         int dataSize = _columnVectorArray.length;
         if (size != dataSize)
         {
             throw new IllegalActionException("The data cloumn didn't match head column");
         }
         Token[] data = new ArrayToken[size];
         for (int i=0; i<size; i++)
         {
             Vector columnData = _columnVectorArray[i];
             Token[] columnTokens = 
                 transformStringVectorToTokenArray(columnData, _selectedColumnTypeList[i]);
             //transform columnTokens to ArrayToken
             ArrayToken arrayToken = new ArrayToken(columnTokens);
             data[i] = arrayToken;            
         }
         _recordToken = new RecordToken(_selectedColumnLabelList, data);
     }
     
     /*
      * Initialize as byteArray. It will read byte array from reader
      */
     private void initializeAsByteArray() throws IllegalActionException
     {
          _reader = _selectedCachedDataItem.getDataInputStream();
          if (_reader != null)
          {
               try 
               {
                  //_reader.reset();
                  _numberOfBytesRead = _reader.read(_byteArray);
               }
               catch (IOException ex) 
               {
                  throw new IllegalActionException(this, ex,
                                            "couldn't read inputstream for as byte array output.");
               }
 
          }
     }
     
     /*
      * InitializeAsUnCompressFileName. It will read the file path from
      * uncompressed directory
      */
     private void initializeAsUnCompressedFileName() throws IllegalActionException
     {
         if (_selectedTableEntity.getHasZipDataFile() || _selectedTableEntity.getHasGZipDataFile() ||
         _selectedTableEntity.getHasGZipDataFile())
         {
           String targetFileExtension = _fileExtensionInZip.stringValue();
           dbg.print("The file extension will send out is "+ targetFileExtension, 2);
           EcogridCompressedDataCacheItem compressedItem = (EcogridCompressedDataCacheItem)_selectedCachedDataItem; 
           _targetFilePathInZip = compressedItem.getUnzippedFilePath(targetFileExtension);
         }
         else
         {
             throw new IllegalActionException("The selected entity is not a compressed data file");
         }
     }
     
     /*
      * InitializeAsCacheFileName. It will read the local cache file name
      */
     private void initializeAsCacheFileName() throws IllegalActionException
     {
          if (_selectedCachedDataItem != null)
          {
            _selectedDataCacheItemLocalFileName = _selectedCachedDataItem.getLocalFileName();
            dbg.print("The locale file name for data cache is "+ _selectedDataCacheItemLocalFileName, 2);    
            _selectedDataCacheItemResourceName  = _selectedCachedDataItem.getResourceName();
            dbg.print("The resource name for the data cache is "+_selectedDataCacheItemResourceName, 2);
          }
          else
          {
            throw new IllegalActionException("The selected entity is null and couldn't find data cache for it");
          }
     }
     
    /**
     * @return Description of the Returned Value
     * @exception IllegalActionException
     *                Description of Exception
     * @since
     */
    public boolean prefire() throws IllegalActionException
    {
        return super.prefire();
    }

    /*
     * Send an entire tableas a single token over the port on each fire event.
     * 
     * @exception IllegalActionException
     *                If there is no director.
     */
    private void fireAsTable() throws IllegalActionException
    {
        TypedIOPort pp = (TypedIOPort) this.getPort("DataTable");
        pp.send(0, new StringToken(new String(_selectedCachedDataItem.getData())));

        pp = (TypedIOPort) this.getPort("Delimiter");
        String delim = _selectedTableEntity.getDelimiter();
        pp.send(0, new StringToken(delim));

        pp = (TypedIOPort) this.getPort("numColumns");
        pp.send(0, new IntToken(_selectedTableEntity.getAttributes().length));

    }

    /*
     * Send a row as a single token over the port on each fire event.
     * 
     * @exception IllegalActionException
     *                If there is no director.
     */
    private void fireAsRow() throws IllegalActionException
    {
        
        TypedIOPort port = (TypedIOPort) this.getPort(ROWDATAPORTNAME);
        if (port != null) {
            Vector rowVector = null;
            try
            {
              rowVector = gotRowVectorFromSource();
            }
            catch (Exception e)
            {
              throw new IllegalActionException(e.getMessage());
            }
            int size = rowVector.size();
            Token[] tokenArray = new Token[size];
            for (int i=0; i<size; i++)
            {
              String eleStr = (String)rowVector.elementAt(i);
              Vector singleVector = new Vector();
              singleVector.add(eleStr);
              Token[] singleArray = transformStringVectorToTokenArray
                                      (singleVector,_selectedColumnTypeList[i]);
              tokenArray[i] = singleArray[0];
             
            }
            RecordToken rowRecordToken = new RecordToken(_selectedColumnLabelList, tokenArray);
            port.send(0, rowRecordToken);

            //_recordCount++;
        }
    }

    /**
     * Send a record's tokens over the ports on each fire event.
     * 
     * @exception IllegalActionException
     *                If there is no director.
     */
    private void fireAsField() throws IllegalActionException
    {
        //dbg.print("Processing record: " + i, 2);
        Enumeration colEnum = _columns.elements();
        //Enumeration dataEnum = _dataVectors[_recordCount].elements();
        Vector rowData = null;
        try
        {
          rowData = gotRowVectorFromSource();
        }
        catch (Exception e)
        {
          throw new IllegalActionException(e.getMessage());
        }
        //Enumeration typeEnum = _colTypes.elements();
        int vectorIndex = 0;
        while (colEnum.hasMoreElements() && vectorIndex < rowData.size()) {
           
            DSTableFieldIFace colDef = (DSTableFieldIFace) colEnum
                    .nextElement();
            //String eleStr = (String) dataEnum.nextElement();
            String eleStr =(String) rowData.elementAt(vectorIndex);
            vectorIndex++;
            Type type = (Type) getBaseType(colDef.getDataType());
            TypedIOPort port = (TypedIOPort) this.getPort(colDef.getName()
                    .trim());
            if (port != null) {
                Token val = null;
                //find the data type for each att
                if (type == BaseType.INT) {
                    try {
                        val = new IntToken(new Integer(eleStr).intValue());
                    } catch (Exception e) {
                        val = new IntToken(0);
                    }
                } else if (type == BaseType.DOUBLE) {
                    val = new DoubleToken(new Double(eleStr).doubleValue());
             
                } else if (type == BaseType.LONG) {
                    val = new LongToken(new Long(eleStr).longValue());

                } else if (type == BaseType.STRING) {
                    val = new StringToken(eleStr);

                } else {
                    val = new IntToken(0);
                }
                //send the data on the port
                port.send(0, val);
            }
        }
        //_recordCount++;
    }
    
    /*
     *  Transform a column vector to  array token, and send the array token
     *  to port in one fire
     * @exception IllegalActionException
     *                If there is no director.
     */
    private void fireAsColumnVector() throws IllegalActionException
    {
        //dbg.print("Processing record: " + i, 2);
        if (_columnVectorArray != null)
        {
            Enumeration colEnum = _columns.elements();
            int rowSize = _columnVectorArray.length;
            //Enumeration typeEnum = _colTypes.elements();
            int rowIndex = 0;
            while (colEnum.hasMoreElements() && rowIndex < rowSize) 
            {
                DSTableFieldIFace colDef = (DSTableFieldIFace) colEnum
                        .nextElement();
                Vector columnVector = _columnVectorArray[rowIndex];
                rowIndex++;
                Type type = (Type) getBaseType(colDef.getDataType());
                TypedIOPort port = (TypedIOPort) this.getPort(colDef.getName()
                        .trim());
                if (port != null) 
                {
                   
                    Token[] columnToken    = 
                        transformStringVectorToTokenArray(columnVector, type);
                    //send the data on the port
                    port.send(0, new ArrayToken(columnToken));
                }
            }  
        }
        else
        {
            throw new IllegalActionException("Couldn't get source data");
        }
    
    }
    
    /*
     * This method will be called in fire method when user choose the output type
     * is As Byte Array. It is handle large binary file. During the excution,
     * input tream of from dataChacheItem will be first read into a byte array in
     * initilize method(Only once for start during a excution). Then fire method
     * will send out the byte array. Post fire method will continue read the input
     * stream to byte array and fire method will send them out. This will stop
     * util reach end of inputstream.(Output the data read in the initialize() 
     * or in the previous invocation of postfire(), if there is any.
     * 
     */
    private void fireAsByteArray() throws IllegalActionException
    {
      if (_numberOfBytesRead > 0) {
            Token _bytes[] = new Token[_numberOfBytesRead];
            for (int i = 0; i < _numberOfBytesRead; i++) {
                _bytes[i] = new UnsignedByteToken(_byteArray[i]);
               
            }
            TypedIOPort dataPort = (TypedIOPort)this.getPort(_BINARYDATAPORT);
            dataPort.send(0, new ArrayToken(_bytes));
        }

    }
    
    /*
     * A zipped cache data item will be unzipped into a directory where it stores
     * a file list. Given a file extension,( which will be read from string attribute
     * _fileExtensionInZip) we will get array of file path. So each fire, a string
     * (file name )will be send out.
     */
    private void fireAsUnzippedFileName() throws IllegalActionException
    {
  
      if (_targetFilePathInZip != null)
      {
        //put file name into a string token array
        int length = _targetFilePathInZip.length;
        StringToken [] fileNameArray = new StringToken[length];
        for (int i=0; i<length; i++)
        { 
           
           fileNameArray[i] = new StringToken(_targetFilePathInZip[i]);
        }
        TypedIOPort pp = (TypedIOPort)this.getPort(_UNZIPPEDFILENAMEPORT);
        pp.send(0, new ArrayToken(fileNameArray));
        _targetFilePathInZip = null;
        
      }
    }
    
    /*
     * This method will reader a row vector from data source, either from
     * resultset which excuted by data query or delimiterdReader which reader
     * from data inputtream - _reader. This method will be called in asFired**
     * method
     */
     private Vector gotRowVectorFromSource() throws Exception
     {
       Vector rowVector = new Vector();
       if (_resultSet != null)
       {
         ResultSetMetaData metadata = _resultSet.getMetaData();
         int columnSize = metadata.getColumnCount();

         if(_resultSet.next()) 
         {
           for (int i = 0; i < columnSize; i++) 
           {
               String str = _resultSet.getString(i + 1);
               rowVector.add(str);
           }
         }
       }
       else if (_reader != null && (!_hasSQLCommand || _numberOfEntities==1))
       {
          if (_selectedTableEntity.isSimpleDelimited())
          {
            rowVector = _simpleDelimitedReader.getRowDataVectorFromStream();
          }
          else
          {
             rowVector = _complexFormatReader.getRowDataVectorFromStream();
          }
          
       }
       
       //if the row vector is empty, this means it reach end of stream
       if (rowVector.isEmpty())
       {
         _endOfStream = true;
       }
       return rowVector;
     }
     
    
    /*
     * This method will send out a cache local file path and resource location
     * of this data object
     */
    private void fireAsCacheFileName() throws IllegalActionException
    {
      
      TypedIOPort pp = (TypedIOPort)this.getPort(_CACHELOCALFILENAMEPORT);
      pp.send(0, new StringToken(_selectedDataCacheItemLocalFileName));
      TypedIOPort pp1 = (TypedIOPort)this.getPort(_CACHERESOURCENAMEPORT);
      pp1.send(0, new StringToken(_selectedDataCacheItemResourceName));
      _selectedDataCacheItemLocalFileName = null;
      _selectedDataCacheItemResourceName  = null;
      
    }
    
    /*
     * This method will fire as column based record. Column based record
     * is already prepared in initialize
     */
    private void fireAsColumnBasedRecord() throws IllegalActionException
    {
        TypedIOPort pp = (TypedIOPort)this.getPort(RECORDPORTNAME);
        pp.send(0, _recordToken);
    }
    
    /**
     * Send a record's tokens over the ports on each fire event.
     * 
     * @exception IllegalActionException
     *                If there is no director.
     */
    public void fire() throws IllegalActionException
    {
        //dbg.print("In fire method", 2);
        super.fire();
        switch (_outputType) {
        case _ASTABLE:
            fireAsTable();
            break;
        case _ASROW:
            fireAsRow();
            break;
        case _ASFIELD:
            fireAsField();
            break;
        case _ASBYTEARRAy:
            fireAsByteArray();
            break;
        case _ASUNZIPPEDFILENAME:
           fireAsUnzippedFileName();
           break;
        case _ASCACHEFILENAME:
             fireAsCacheFileName();
             break;
        case _ASCOLUMNVECTOR:
            fireAsColumnVector();
            break;
        case _ASCOLUMNBASEDRECORD:
            fireAsColumnBasedRecord();
            break;
        default:
            throw new IllegalActionException(this, "Unrecognized outputType: "
                    + _outputType);

        }
    }
    
  /** 
    * This method is only for output as byte array. 
    * Read the next bytes from the input stream into array. Fire method will send
    *  the array out. If there reached EOF ,
    *  return false.  Otherwise, return whatever the superclass returns.
    *  @exception IllegalActionException If there is a problem reading
    *   the file.
    */
   public boolean postfire() throws IllegalActionException {
     if(_outputType == _ASBYTEARRAy){
       if (_reader == null) {
         return false;
       }
       try {
         TypedIOPort endOfStream = (TypedIOPort)getPort(_ENDOFSTREAMPORT);
         _numberOfBytesRead = _reader.read(_byteArray);
         if (_numberOfBytesRead <= 0) {
           // it is reach the end of inputstream;
           endOfStream.broadcast(BooleanToken.TRUE);
           return false;
         }
         endOfStream.broadcast(BooleanToken.FALSE);
         return super.postfire();
       }
       catch (IOException ex) {
         throw new IllegalActionException(this, ex, "Postfire failed");
       }
     }
     else if(_outputType == _ASUNZIPPEDFILENAME)
     {
       if (_targetFilePathInZip == null)
       {
         return false;
       }
       else
       {
           return super.postfire();
       }
     }
     else if(_outputType == _ASCACHEFILENAME)
     {
       if (_selectedDataCacheItemLocalFileName == null || _selectedDataCacheItemResourceName == null )
      {
        return false;
      }
      else
      {
          return super.postfire();
      }

     }
     else if (_outputType == _ASFIELD || _outputType == _ASROW 
             || _outputType == _ASCOLUMNVECTOR || _outputType == _ASCOLUMNBASEDRECORD)
     {
       if (_endOfStream)
       {
         return false;
       }
       else
       {
         return super.postfire();
       }
     }
     else if (_outputType == _ASTABLE)
     {
       return false;
     }
     else
     {
         return super.postfire();
     }
   }
   
 
 
 
   /** 
    *  This method is only for output as byte array. In this method it will reset
    *  the _reader to null and get datacached item
    *  @exception IllegalActionException If an IO error occurs.
    */
   public void wrapup() throws IllegalActionException {
     
     if (_outputType == _ASBYTEARRAy )
     {
       _reader = null;
       //getChachedDataItem(_selectedTableEntity);
     }
     
   }
   
 
    /**
     * 
     * @throws ptolemy.kernel.util.IllegalActionException
     */
    private void initializePort(String aPortName, String aType, 
                                boolean aIsNew, boolean isVector)
            throws ptolemy.kernel.util.IllegalActionException
    {
        // Create a new port for each Column in the resultset
        try {
            String columnName = aPortName.trim();
            TypedIOPort port;
            if (aIsNew) {
                // Create a new typed port and add it to this container
                port = new TypedIOPort(this, columnName, false, true);
                dbg.print("Creating port [" + columnName + "]" + this, 2);
            } else {
                // Set the type of the existing port
                port = (TypedIOPort) this.getPort(columnName);
                dbg.print("Existing port port [" + columnName + "]"
                        + this, 2);
            }
            if (!isVector)
            {
                port.setTypeEquals(getBaseType(aType));
            }
            else
            {
                port.setTypeEquals(new ArrayType(getBaseType(aType)));
            }

        } catch (ptolemy.kernel.util.NameDuplicationException nde) {
            throw new IllegalActionException("One or more attributes has the "
                    + "same name.  Please correct this and try again.");
        }
    }

    private boolean isAlreadyConfigured(int aType)
            throws ptolemy.kernel.util.IllegalActionException
    {
        switch (_outputType) {
        case _ASTABLE:
            return getPort("DataTable") != null && getPort("Delimiter") != null
                    && getPort("numColumns") != null;
          

        case _ASROW:
            return getPort(ROWDATAPORTNAME) != null;
            //return false;

        case _ASFIELD:
            return false; // don't know
        case _ASBYTEARRAy:
           return getPort(_BINARYDATAPORT) != null && getPort(_ENDOFSTREAMPORT) != null;
        case _ASUNZIPPEDFILENAME:
          return getPort(_UNZIPPEDFILENAMEPORT) != null;
        case _ASCACHEFILENAME:
          return getPort(_CACHELOCALFILENAMEPORT) != null && getPort(_CACHERESOURCENAMEPORT) != null;
        case _ASCOLUMNBASEDRECORD:
          return getPort(RECORDPORTNAME) != null;
        default:
            throw new IllegalActionException(this, "Unrecognized outputType: "
                    + _outputType);

        }
    }

    /**
     * Remove all ports by setting their container to null. As a side effect,
     * the ports will be unlinked from all relations. This method is
     * write-synchronized on the workspace, and increments its version number.
     */
    public void removeAllOutputPorts()
            throws ptolemy.kernel.util.IllegalActionException
    {
       Vector empty = new Vector();
       removeOtherOutputPorts(empty);

    }
    
    /*
     * Remove all ports which's name is not in the selected vector
     */
    private void removeOtherOutputPorts(Vector nonRemovePortName) throws IllegalActionException
    {
        Vector portNames = getExistingPortNames();
        for (Enumeration e = portNames.elements(); e.hasMoreElements();) {
            String currPortName = (String) e.nextElement();
            if (!nonRemovePortName.contains(currPortName))
            {
              TypedIOPort port = (TypedIOPort)this.getPort(currPortName);
              if (port != null && !port.isInput()) {
                try {
                  port.setContainer(null);
                }
                catch (Exception ex) {
                  throw new IllegalActionException(this,
                                                   "Error removing port: " +
                                                   currPortName);
                }
      
              }
            }
        }

    }

    /**
     * It reconfigures all the Ports to represent passing data back a row at a
     * time.
     * 
     * @throws ptolemy.kernel.util.IllegalActionException
     */
    private void reconfigurePortsAsTable()
            throws ptolemy.kernel.util.IllegalActionException
    {
        if (isAlreadyConfigured(_outputType))
        {
          Vector nonRemovePorts = new Vector();
          nonRemovePorts.add("DataTable");
          nonRemovePorts.add("Delimiter");
          nonRemovePorts.add("numColumns");
          removeOtherOutputPorts(nonRemovePorts);
          TypedIOPort pp1 = (TypedIOPort)this.getPort("DataTable");
          pp1.setTypeEquals(BaseType.STRING);
          TypedIOPort pp = (TypedIOPort)this.getPort("Delimiter");
          pp.setTypeEquals(BaseType.STRING);
          TypedIOPort pp2 = (TypedIOPort)this.getPort("numColumns");
          pp2.setTypeEquals(BaseType.INT);
          return;

        }

        try {
            removeAllOutputPorts();
            TypedIOPort pp = new TypedIOPort(this, "DataTable", false, true);
            pp.setTypeEquals(BaseType.STRING);

            pp = new TypedIOPort(this, "Delimiter", false, true);
            pp.setTypeEquals(BaseType.STRING);

            pp = new TypedIOPort(this, "numColumns", false, true);
            pp.setTypeEquals(BaseType.INT);
        } catch (Exception ex) {
            throw new IllegalActionException(this,
                    "Error creating Table Ports ");
        }
    }

    /**
     * It reconfigures all the Ports to represent passing data back a row at a
     * time.
     * 
     * @throws ptolemy.kernel.util.IllegalActionException
     */
    private void reconfigurePortsAsRow(Vector aNewColumns)
            throws ptolemy.kernel.util.IllegalActionException
    {
        generteLabelListAndTypteListFromColumns(aNewColumns);
        // the record is an ArrayToken, we should transfer from base type to
        // array type
        if (_selectedColumnTypeList != null)
        {
            int size = _selectedColumnTypeList.length;
            Type[] typeList = new Type[size];
            for (int i = 0; i < size; i++)
            {
                typeList[i] = _selectedColumnTypeList[i];
            }
            RecordType recordType = new RecordType(_selectedColumnLabelList,
                    typeList);
            if (isAlreadyConfigured(_outputType))
            {
                TypedIOPort pp1 = (TypedIOPort) this.getPort(ROWDATAPORTNAME);
                pp1.setTypeEquals(recordType);
                return;
            }
            try
            {
                removeAllOutputPorts();
                TypedIOPort pp1 = new TypedIOPort(this, ROWDATAPORTNAME, false,
                        true);
                pp1.setTypeEquals(recordType);
            }
            catch (Exception ex)
            {
                throw new IllegalActionException(this,
                        "Error creating row data port ");
            }
        }
       
    }

    /**
     * It reconfigures all the Ports from the new "columns" list Meaning it
     * removes port not in the list and adds new port that are in the list but
     * not yet there
     * 
     * @throws ptolemy.kernel.util.IllegalActionException
     */
    private void reconfigurePortsAsField(Vector aNewColumns)
            throws ptolemy.kernel.util.IllegalActionException
    {
        boolean isArrayToken = false;
        configurePortsAsField(aNewColumns, isArrayToken);
    }
    
  
  
    /**
     * It reconfigures all the Ports from the new "columns" list Meaning it
     * removes port not in the list and adds new port that are in the list but
     * not yet there
     * 
     * @throws ptolemy.kernel.util.IllegalActionException
     */
    private void reconfigurePortsAsColumnVector(Vector aNewColumns)
            throws ptolemy.kernel.util.IllegalActionException
    {
        boolean isArrayToken = true;
        configurePortsAsField(aNewColumns, isArrayToken);
    }
    
    /*
     * This method will configure ports for every selected fields(attributes)
     * If the output is for a value as a token, isArrayToken = false.
     * If the ouput is a vector a column data, isArrayToken = true;
     */
    private void configurePortsAsField(Vector aNewColumns, boolean isArrayToken)
            throws ptolemy.kernel.util.IllegalActionException
    {
        if (aNewColumns == null || aNewColumns.isEmpty())
        {
            dbg.print("The cloumns info is null and coudldn't conigure ports as field",2);
            return;
        }
        // Get List of new column names
        Vector newColNames = new Vector();
        for (Enumeration e = aNewColumns.elements(); e.hasMoreElements();)
        {
            newColNames.addElement(((DSTableFieldIFace) e.nextElement())
                    .getName());
        }

        // Get List of Existing Port names
        Vector currentNames = getExistingPortNames();

        // loop through the current column names and delete any "Current"
        // name that isn't in the list of "new" names
        for (Enumeration e = currentNames.elements(); e.hasMoreElements();)
        {
            String currPortName = (String) e.nextElement();
            TypedIOPort port = (TypedIOPort) this.getPort(currPortName);
            if (port != null && !newColNames.contains(currPortName)
                    && !port.isInput())
            {
                try
                {
                    port.setContainer(null);
                }
                catch (Exception ex)
                {
                    throw new IllegalActionException(this,
                            "Error removing port: " + currPortName);
                }
            }
        }

        // Now add all the new columns
        for (Enumeration e = aNewColumns.elements(); e.hasMoreElements();)
        {
            DSTableFieldIFace column = (DSTableFieldIFace) e.nextElement();
            String name = column.getName();
            boolean isNew = !currentNames.contains(name);
            initializePort(name, column.getDataType(), isNew, isArrayToken);
        }
    }
    
    /*
     * This method will configure the actor output as a column based record.
     * It will has one output port and name will be "Record".
     */
    private void reconfigurePortsAsColumnBasedRecord(Vector aNewColumns)
                     throws ptolemy.kernel.util.IllegalActionException
    {
        generteLabelListAndTypteListFromColumns(aNewColumns);
        // the record is an ArrayToken, we should transfer from base type to
        // array type
        if (_selectedColumnTypeList != null)
        {
        int size = _selectedColumnTypeList.length;
            ArrayType[] arrayTypeList = new ArrayType[size];
            for (int i = 0; i < size; i++)
            {
                arrayTypeList[i] = new ArrayType(_selectedColumnTypeList[i]);
            }
            RecordType recordType = new RecordType(_selectedColumnLabelList,
                    arrayTypeList);
            if (isAlreadyConfigured(_outputType))
            {
                TypedIOPort pp1 = (TypedIOPort) this.getPort(RECORDPORTNAME);
                pp1.setTypeEquals(recordType);
                return;
            }
            try
            {
                removeAllOutputPorts();
                TypedIOPort pp1 = new TypedIOPort(this, RECORDPORTNAME, false,
                        true);
                pp1.setTypeEquals(recordType);
            }
            catch (Exception ex)
            {
                throw new IllegalActionException(this,
                        "Error creating Table Ports ");
            }
        }
    }
  
  
    /*
     * This method will reconfigure ports when user choose the output as
     * byte array. There are two exports in the actor, one is the port for
     * sending out the byte array, the other is send out a boolean indactor
     * if the byte array is reach the end of input stream.
     * @throws ptolemy.kernel.util.IllegalActionException
     */
    private void reconfigurePortsAsByteArray() throws ptolemy.kernel.util.
        IllegalActionException {
       if (isAlreadyConfigured(_outputType))
       {
         //removeAllOutputPorts();
         //reset type
         TypedIOPort pp1 = (TypedIOPort)this.getPort(_BINARYDATAPORT);
         pp1.setTypeEquals(new ArrayType(BaseType.UNSIGNED_BYTE));
         TypedIOPort pp = (TypedIOPort)this.getPort(_ENDOFSTREAMPORT);
         pp.setTypeEquals(BaseType.BOOLEAN);
         return;
       }
       try 
       {
        removeAllOutputPorts();
        TypedIOPort pp1 = new TypedIOPort(this, _BINARYDATAPORT, false, true);
        pp1.setTypeEquals(new ArrayType(BaseType.UNSIGNED_BYTE));
  
        TypedIOPort pp = new TypedIOPort(this, _ENDOFSTREAMPORT, false, true);
        pp.setTypeEquals(BaseType.BOOLEAN);
      }
      catch (Exception ex) 
      {
        throw new IllegalActionException(this,
                                         "Error creating Table Ports ");
      }
    }
    
    /*
     * This method will configure the ports for output the unzip file path.
     * There is only one ports for this and the type is string token
     */
    private void reconfigurePortsAsUnzippedFileName() throws ptolemy.kernel.util.
        IllegalActionException
    {
      if (isAlreadyConfigured(_outputType))
      {
        
        TypedIOPort pp1 = (TypedIOPort)this.getPort(_UNZIPPEDFILENAMEPORT);
        pp1.setTypeEquals(new ArrayType(BaseType.STRING));
        return;
      }
      try 
      {
       removeAllOutputPorts();
       TypedIOPort pp1 = new TypedIOPort(this,_UNZIPPEDFILENAMEPORT, false, true);
       pp1.setTypeEquals(new ArrayType(BaseType.STRING));
     }
     catch (Exception ex) 
     {
       throw new IllegalActionException(this,
                                        "Error creating Table Ports ");
     }

    }
    
    /**
     * This method will reconfig ports for people want to get a cache local file
     * name. There are two ports for this output type. One port is for cache local
     * file name. This is an absolute path of cache file name. The other port is
     * for the resource location( it is a url in eml distribution).
     * @throws IllegalActionException
     */
    private void reconfigurePortsAsCacheLocalFileName() throws IllegalActionException
    {
      if (isAlreadyConfigured(_outputType))
       {
         //removeAllOutputPorts();
         //reset type
         TypedIOPort pp1 = (TypedIOPort)this.getPort(_CACHELOCALFILENAMEPORT);
         pp1.setTypeEquals(BaseType.STRING);
         TypedIOPort pp = (TypedIOPort)this.getPort(_CACHERESOURCENAMEPORT);
         pp.setTypeEquals(BaseType.STRING);
         return;
       }
       try 
       {
        removeAllOutputPorts();
        TypedIOPort pp1 =  new TypedIOPort(this,_CACHELOCALFILENAMEPORT, false, true);
        pp1.setTypeEquals(BaseType.STRING);
        TypedIOPort pp =  new TypedIOPort(this, _CACHERESOURCENAMEPORT , false, true);
        pp.setTypeEquals(BaseType.STRING);

      }
      catch (Exception ex) 
      {
        throw new IllegalActionException(this,
                                         "Error creating ports for cache local file name ");
      }

    }
    /**
     * 
     * @param aNewColumns
     * @param aColTypes
     * @throws ptolemy.kernel.util.IllegalActionException
     */
    private void reconfigurePorts(Vector aNewColumns)
            throws ptolemy.kernel.util.IllegalActionException
    {
       
        //setIconStatus(TITLE_BUSY, RED);

        switch (_outputType) {
        case _ASTABLE:
            reconfigurePortsAsTable();
            break;

        case _ASROW:
            reconfigurePortsAsRow(aNewColumns);
            break;

        case _ASFIELD:
            reconfigurePortsAsField(aNewColumns);
            break;
        case _ASBYTEARRAy:
            reconfigurePortsAsByteArray();
            break;
        case _ASUNZIPPEDFILENAME:
            reconfigurePortsAsUnzippedFileName();
            break;
        case _ASCACHEFILENAME:
              reconfigurePortsAsCacheLocalFileName();
              break;
        case _ASCOLUMNVECTOR:
            reconfigurePortsAsColumnVector(aNewColumns);
            break;
        case _ASCOLUMNBASEDRECORD:
            reconfigurePortsAsColumnBasedRecord(aNewColumns);
            break;
        default:
            throw new IllegalActionException(this, "Unrecognized outputType: "
                    + _outputType);

        }
        //setIconStatus(TITLE_BINARY, YELLOW);
    }

    /**
     * Callback for changes in attribute values.
     */
    public void attributeChanged(ptolemy.kernel.util.Attribute attribute)
            throws ptolemy.kernel.util.IllegalActionException
    {
        //dbg.print("In attribute change method", 2);
        if (attribute == _sqlAttr) {
            String sqlDef = ((Settable) attribute).getDefaultExpression();
            sqlDef = ((Settable) attribute).getExpression();
            if (sqlDef.length() > 0) {
                _hasSQLCommand = true;
                _queryDef = DBQueryDefParserEmitter.parseQueryDef(_schemaDef,
                        sqlDef);
                _columns = _queryDef.getSelects();
                reconfigurePorts(_queryDef.getSelects());
            }

        } else if (attribute == _schemaAttr && !_ignoreSchemaChange) 
            // NOTE: We may skip setting it here because _ignoreSchemaChange
            // may be true
        {
            String schemaDef = ((Settable) _schemaAttr).getExpression();

            // MOML may have a blank definition
            if (schemaDef.length() > 0) {
                _schemaAttrIsSet = true; // remember that we have been set by
                                         // the MOML

                dbg.print("schemaDef >>" + schemaDef + "<<", 2);

                _schemaDef = DBSchemaParserEmitter.parseSchemaDef(schemaDef);
            }

        } else if ((attribute.getName().equals("recordId") || attribute
                .getName().equals("endpoint"))
                && this.hasConnectionValues()
                && !(attribute.getContainer().getContainer() instanceof ResultTreeRoot)) {
            dbg.print("change recorid or endpoints", 2);
            setIconStatus(TITLE_BUSY, RED);
            _cacheMetaDataItem = (EcogridMetaDataCacheItem) DataCacheManager
                    .getCacheItem(this,
                            "MetaData " + getRecordId(), getEndpoint(),
                            EcogridMetaDataCacheItem.class.getName());
            _cacheMetaDataItem.setHasListener(true);
            _cacheMetaDataItem.setEndPoint(getEndpoint());
            _cacheMetaDataItem.setRecordId(getRecordId());
            _cacheMetaDataItem.start();
           

        } else if (attribute == _outputTypeAttr) {
            String strOutputType = _outputTypeAttr.stringValue();
            if (strOutputType.equals("As Table") && _outputType != _ASTABLE) {
                _outputType = _ASTABLE;
                reconfigurePorts(_columns);

            } else if (strOutputType.equals("As Row") && _outputType != _ASROW) {
                _outputType = _ASROW;
                reconfigurePorts(_columns);

            } else if (strOutputType.equals("As Field") && _outputType != _ASFIELD) {
                _outputType = _ASFIELD;
                if (_columns != null && _columns.size() > 0) {
                    reconfigurePorts(_columns);
                }
            } else if (strOutputType.equals(_ASBYTEARRAYLABEL)
                      && _outputType != _ASBYTEARRAy) {
                  // output type will be a byte array and recconfigure ports
                  _outputType = _ASBYTEARRAy;
                  reconfigurePorts(_columns);                
            } else if (strOutputType.equals(_ASUNZIPPEDFILENAMELABEL)
                         && _outputType != _ASUNZIPPEDFILENAME) {
                     // output type will be unzipped file name and recconfigure ports
                     _outputType = _ASUNZIPPEDFILENAME;
                     reconfigurePorts(_columns);                
            
           } else if (strOutputType.equals(_ASCACHEFILENAMELABEL)
                       && _outputType != _ASCACHEFILENAME) {
                   // output type will be unzipped file name and recconfigure ports
                   _outputType = _ASCACHEFILENAME;
                  reconfigurePorts(_columns);          

           } else if (strOutputType.equals(_ASCOLUMNVECTORLABEL)
                   && _outputType != _ASCOLUMNVECTOR) {
               // output type will be unzipped file name and recconfigure ports
               _outputType = _ASCOLUMNVECTOR;
               reconfigurePorts(_columns);          

           } else if (strOutputType.equals(_ASCOLUMNBASEDRECORDLABEL)
                   && _outputType != _ASCOLUMNBASEDRECORD) {
               // output type will be unzipped file name and recconfigure ports
               _outputType = _ASCOLUMNBASEDRECORD;
               reconfigurePorts(_columns);          

           } /*else {
                //throw new IllegalActionException(this,
                //        "Unrecognized outputType: " + strOutputType);
            }*/

        }else if (attribute == _selectedEntityAttr){
            //reset selected entity
             setSelectedEntityValue(true);
          
         
        }
        
       
    }

    /**
     * Creates the schema definition from the incoming data columns
     */
    private void createSchemaFromData(Entity tableEntity)
    {
        try // XXX shouldn't be catch this exception here
        {

            // _schemaAttrIsSet gets set when the schema has come from the MOML
            // So if it is false here, then we set the attribute from the data.
            //
            // If after this the attr gets set from the MOML it will override
            // what we set here.
            //Entity tableEntity = dataItem.getEntity();
            DataCacheObject dataItem = tableEntity.getDataCacheObject();
            if (tableEntity != null) {
                 _schema.addTable(tableEntity);
                 _schemaDef = _schema;

                DBTableNameResolver nameResolver = new DBTableNameResolver();
                try {
                    tableEntity = nameResolver.resolveTableName(tableEntity);
                } catch (Exception e) {
                }
                boolean refresh = false;
                if (tableEntity.isSimpleDelimited())
                {
                    // for simple dimilter, we can use text table
                    _tableGenerator = 
                        new DBTablesGenerator(tableEntity, dataItem.getFileName(),refresh); 
                }
                else
                {
                    // for complex format, we should use inpustream to create table
                    InputStream input = dataItem.getDataInputStream();
                    _tableGenerator = new DBTablesGenerator(tableEntity, input, refresh);
                }
                
                _tableGenerator.run(); // don't do thread

                String schemaDefXML = DBSchemaParserEmitter.emitXML(_schemaDef);
                _ignoreSchemaChange = true;
                ((Settable) _schemaAttr).setExpression(schemaDefXML);
                _ignoreSchemaChange = false;
            }
        } catch (IllegalActionException e) {
            dbg.print("In createSchemaFromData: " + e, 2);
        }

    }

  
    /**
     * Get a URL pointer to the documentation for this data source. The XML 
     * source of the EML document is retrieved from the cache, and then passed 
     * to an XSLT parser to be transformed into HTML format, which is saved in 
     * a temporary file. The URL of the temporary file containing the HTML 
     * result is returned.
     * 
     * @return URL the URL of the HTML file containing the documentation
     */
    public URL getDocumentation()
    {
        String htmlFileName = getRecordId();
        String namespace = getNamespace();
        if (namespace == null)
        {
            namespace = EML2MetadataSpecification.EML200NAMESPACE;
        }
        dbg.print("The name space is "+namespace, 2);
        URL htmlUrl = null;
        // Get the metadata XML document and transform it to html
        if (_cacheMetaDataItem.isReady()) {
            try {
                InputStream is = _cacheMetaDataItem.getDataInputStream();
                InputStreamReader source = new InputStreamReader(is);
                htmlUrl = Util.getMetadataHTMLurl(source, namespace, htmlFileName);
               
            } catch (Exception fnfe) {
                dbg.print("Could not open temporary output file.", 2);
            }
        }
        return htmlUrl;
    }
    
   

    /**
     * Creates the delimtedReader from the data item inputstream
     */
    private void createDelimitedReader() throws Exception
    {
        /*String data = aDataStr.toString();
        dbg.print("-----------------\n" + data
                       + "\n-----------------\n", 2);*/

        //dbg.print("entityhash: " + entityhash.toString(), 2);
        if (_selectedTableEntity.isSimpleDelimited())
        {
          boolean stripHeaderLine = true;
          _simpleDelimitedReader = new DelimitedReader(_reader, _selectedTableEntity
                .getAttributes().length, _selectedTableEntity.getDelimiter(),
                _selectedTableEntity.getNumHeaderLines(), _selectedTableEntity.getRecordDelimiter(), 
                _selectedTableEntity.getNumRecords(), stripHeaderLine);
        }
        else
        {
            _complexFormatReader = new TextComplexFormatDataReader(_reader, _selectedTableEntity);
        }

        //_dataVectors = dr.getTokenizedData(true);
    }

    /**
     * returns the names of the current ports
     */
    private Vector getExistingPortNames()
    {
        List l = this.portList();
        Vector v = new Vector();
        for (int i = 0; i < l.size(); i++) {
            TypedIOPort p = (TypedIOPort) l.get(i);
            v.add(p.getName().trim());
        }
        return v;
    }

    /**
     * Returns a Ptolemly type for a given Kepler DataSource Type
     * @param aType DataSource type
     * @return Ptolemy type
     */
    private BaseType getBaseType(String aType)
    {
        BaseType type = (BaseType) _TypeHash.get(aType);
        if (type == null) { return BaseType.UNKNOWN; }
        return type;
    }

    /**
     * Fills the BaseType Vector with a BaseType for each column
     * @param aCols the columns of the DB
     * @param aBaseTypes the Vecotr to be filled
     */
    private void buildBaseTypeVectorFromColumns(Vector aCols, Vector aBaseTypes)
    {
        aBaseTypes.clear();
        for (Enumeration e = aCols.elements(); e.hasMoreElements();) {
            DSTableFieldIFace column = (DSTableFieldIFace) e.nextElement();
            aBaseTypes.addElement(getBaseType(column.getDataType()));
        }

    }

    /*
     * Method to transform resultset to a vector array data. A vector is a row
     * and data type is string.
     */
    private Vector[] transformResultSet(ResultSet rs) throws SQLException
    {
        Vector[] dataArray = null;
        Vector data = new Vector();
        ResultSetMetaData metadata = rs.getMetaData();
        int columnSize = metadata.getColumnCount();

        while (rs.next()) {
            Vector rowData = new Vector();
            for (int i = 0; i < columnSize; i++) {
                String str = rs.getString(i + 1);
                rowData.add(str);
            }
            data.add(rowData);
        }
        dataArray = transformVectorToArray(data);
        return dataArray;
    }

    /**
     * 
     * @param vector
     * @return
     */
    private Vector[] transformVectorToArray(Vector vector)
    {
        if (vector == null) { return null; }
        int size = vector.size();
        Vector[] array = new Vector[size];
        for (int i = 0; i < size; i++) {
            array[i] = (Vector) vector.elementAt(i);
        }
        return array;
    }

    //------------------------------------------------------------------------
    //-- DataCacheListener
    //------------------------------------------------------------------------

    public void complete(DataCacheObject aItem)
    {
        dbg.print("complete: " + this, 2);

        aItem.removeListener(this);
        aItem.setHasListener(false);
        if (aItem.isReady()) 
        {
           if (aItem instanceof EcogridDataCacheItem) 
           {
              
               EcogridDataCacheItem item = (EcogridDataCacheItem)aItem;
               String entityIdentifier = item.getEntityIdentifier();
               int index = lookupEntityListName(entityIdentifier);
               Entity entity = null;
               if (index != INDEXFORNOTFOUND)
               {
                   entity = (Entity) _entityList.elementAt(index);
               }
               entity.setDataCacheObject(item);    
               if (!entity.getIsImageEntity()) {
                 
                    createSchemaFromData(entity);
                    
                 
                }
                _numberOfDownloadEntities++;

            } else if (aItem instanceof EcogridMetaDataCacheItem) {
                try {
                    parsePackage(new InputStreamReader(new FileInputStream(new File(aItem.getLocalFileName()))));
                } catch (Exception e) {
                    System.err.println(e);
                    e.printStackTrace();
                    setIconStatus(TITLE_ERROR, MAGENTA); 
                }
                
            }
            
            DataCacheManager.getInstance().saveCache();
            
        }
        else if (aItem.getStatus() == aItem.CACHE_ERROR)
        {
            //this is for failed download
            if (aItem instanceof EcogridDataCacheItem) 
            {
               dbg.print("In failed download path", 2);
               EcogridDataCacheItem item = (EcogridDataCacheItem)aItem;
               String entityIdentifier = item.getEntityIdentifier();
               int index = lookupEntityListName(entityIdentifier);
               Entity entity = null;
               if (index != INDEXFORNOTFOUND)
               {
                   entity = (Entity) _entityList.elementAt(index);
               }
               //because fail to download, set null as data cache item
               entity.setDataCacheObject(null);
               _numberOfDownloadEntities++;
               _numberOfFailedDownloadEntities++;
               
            }
        }
        
      
    }
    
    /*
     * parses the package and fills in the names and types arrays
     */
    private void parsePackage(Reader eml) throws IllegalActionException
    {
        Eml200Parser eml2parser;

        try { //parse the package for the names and types of the atts
            dbg.print("creating parser", 2);
            eml2parser = new Eml200Parser();
            dbg.print("parsing...", 2);
            eml2parser.parse(new InputSource(eml));
            // get if this package has image entity
            //_hasImageEntity = eml2parser.getHasImageEntity();
            //dbg.print("This pakcage has image entity " + _hasImageEntity, 2);
            dbg.print("Done parsing", 2);
        } catch (Exception e) {
            throw new IllegalActionException("Error parsing the eml package: "
                    + e.getMessage());
        }

        getData(eml2parser); // fills in the _dataVectors data member
        setOptionsForSelectedEntityAttribute();
        // after download all data cache(not matter success or fail) we need to set up
        //selectedEntity cloumns and change the icon color(-reader will be set
        // in initialize()
        // here is waiting dowload finsihed
         while (_numberOfDownloadEntities != _numberOfEntities)
         {
             //System.out.println("waiting!!!!!!!!!!!!!");
         }
                   
         try
         {
             // this method will set up cloumns info if not sql command
             setSelectedEntityValue(false);
                                   
         }
         catch(Exception e)
         {
             dbg.print("The error in set up selected entity is "+e.getMessage(), 2);
                
          }
         //setOptionsForSelectedEntityAttribute();
         if (_numberOfFailedDownloadEntities == 0)
         {
           setIconStatus(TITLE_BINARY, YELLOW);
         }
    }

    /**
     * get the data based on the contents of the package
     * 
     * @param parser
     *            the parser that has already parsed the package.
     */
    private void getData(Eml200Parser parser)
            throws IllegalActionException
    {
        if (parser != null)
        {
            _numberOfEntities = parser.getEntityCount();
            /*if (parser.getEntityCount() > 1) {
                throw new IllegalActionException(
                        "Currently this parser only deals with one entity. "
                        + "Please use a package with only one entity.");*/
            if ( _numberOfEntities == 0) 
            {
                throw new IllegalActionException(
                        "There must be at least one entity in the EML package.");
            }
            Hashtable entityList = parser.getEntityHash();
            // initialize selected entity
            _selectedTableEntity = (Entity) entityList.get(entityList.keys().nextElement());
            //dbg.print("entityhash: " + _entityList.toString(), 2);

            // start a thread to get data chachedItem
            Enumeration enu = entityList.elements();
            while(enu.hasMoreElements())
            {
                  
                Entity singleEntity = (Entity)enu.nextElement();
                _entityList.add(singleEntity);
                getChachedDataItem(singleEntity);
            }
            
            
           
        }
    }
    
    /*
     * This method will start a thread to get chached data item base on the url
     * getting from table entity. This method will be call in method getdatac and
     * wrapup
     */
    private void getChachedDataItem(Entity tableEntity)
    {
      if (tableEntity == null)
      {
        dbg.print("The table enity is null and couldn't get chached data item", 2);
        return;
      }
      String fileURLStr = tableEntity.getURL();
      dbg.print("Data URL is: " + fileURLStr, 2);
      // we need to distinguish zip file and generate
      //String compression = tableEntity.getCompressionMethod();
      EcogridDataCacheItem cachedDataItem = null;
      if (tableEntity.getHasZipDataFile())
      {
        dbg.print("This is a zip data cacheItem", 2);
        cachedDataItem = (EcogridZippedDataCacheItem) 
            DataCacheManager.getCacheItem(this, "Data", fileURLStr, EcogridZippedDataCacheItem.class.getName());
        //_isZipDataFile = true;
      }
      else if (tableEntity.getHasGZipDataFile())
      {
        dbg.print("This is a gzip data cacheItem", 2);
        cachedDataItem = (EcogridGZippedDataCacheItem) 
            DataCacheManager.getCacheItem(this, "Data", fileURLStr, EcogridGZippedDataCacheItem.class.getName());
        if (tableEntity.getHasTarDataFile())
        {
          dbg.print("This is gizp and tar data cache item", 2);
          cachedDataItem.setIsTarFile(true);
        }
            
      }
      else if (tableEntity.getHasTarDataFile())
      {
        dbg.print("This is a tar data cacheItem", 2);
        cachedDataItem = (EcogridTarArchivedDataCacheItem) 
           DataCacheManager.getCacheItem(this, "Data", fileURLStr, EcogridTarArchivedDataCacheItem.class.getName());
      }
      else
      {
        dbg.print("This is a uncompressed data cacheItem", 2);
        cachedDataItem = (EcogridDataCacheItem) DataCacheManager
            .getCacheItem(this, "Data", fileURLStr,
                          EcogridDataCacheItem.class.getName());
      }
      cachedDataItem.setEndPoint(this.getEndpoint());
      cachedDataItem.setHasListener(true);
      // add entity identifier order to track the cachedata item associate with 
      // which entity(In complete method)
      cachedDataItem.setEntityIdentifier(tableEntity.getName());
      cachedDataItem.start();

    }
    
    /*
     * This method will set options for "Selected Entity Attribute" after get all
     * data entities. The options will be the entities' name
     */
    private void setOptionsForSelectedEntityAttribute()
    {
       if (_entityList != null)
       {
           int length = _entityList.size();
           for (int i=0;  i<length; i++)
           {
               Entity entity = (Entity)_entityList.elementAt(i);
               String entityName = entity.getName();
               if (entityName != null && !entityName.trim().equals(""))
               {
                  _selectedEntityAttr.addChoice(entityName);
               }
               
           }
         
       }
    }
    
    /*
     * Method to set up selected entity. If Attribute "selectedEntity" already
     * has value, look up the enitytList and found the selected entity. If 
     * attribute "selectedEntity doesn't have any value, we choose the index 0
     * as selected entity.
     */
    private void setSelectedEntityValue(boolean fromAttributeChange) throws IllegalActionException
    {
       
       String selectedEntityName = _selectedEntityAttr.stringValue();
       dbg.print("The selected entity name is "+selectedEntityName, 2);
       if (!_entityList.isEmpty())
       { 
           
           if (selectedEntityName != null &&
               !selectedEntityName.trim().equals("")) {
               // already has a selected entity in momol
               dbg.print("There is a selected entity in actor", 2);
               int selectedIndex = lookupEntityListName(selectedEntityName);
               if (selectedIndex == INDEXFORNOTFOUND) {
                   throw new IllegalActionException(
                           "The selected Entity in momol cound't be found");
               } else {
                   _selectedTableEntity = (Entity) _entityList.elementAt(
                           selectedIndex);
                   _selectedCachedDataItem = (EcogridDataCacheItem) 
                                             _selectedTableEntity.getDataCacheObject();
                   
               }
           } else {
               // no selected enity in momol and we need selected one
               dbg.print("There is NOT a selected entity in actor", 2);
               _selectedTableEntity = (Entity) _entityList.elementAt(
                       DEFAULTINDEX);
               _selectedCachedDataItem = (EcogridDataCacheItem)_selectedTableEntity.getDataCacheObject();
                                         
               String entityName = _selectedTableEntity.getName();
               dbg.print("set the default entity name " + entityName +
                         "because the there is no selected entity", 2);
               _selectedEntityAttr.setExpression(entityName);
                 if(!fromAttributeChange)
               {
                  dbg.print("send a moml request for adding selected Entity parameter", 2);
                  StringBuffer buffer = new StringBuffer();
                  buffer.append("<property name=\"");
                  buffer.append(_SELECTEDENTITY);
                  buffer.append("\" class=\"");
                  buffer.append(_selectedEntityAttr.getClassName());
                  buffer.append("\" value=\"");
                  buffer.append(entityName);
                  buffer.append("\"/>");
                  String moml = buffer.toString();
                  dbg.print("The moml string is "+moml, 2);
                  NamedObj container = (NamedObj) this.getContainer();
                  NamedObj composite = (NamedObj) container.getContainer();
                  MoMLChangeRequest request = new MoMLChangeRequest(this, this, moml.toString());
                  request.setUndoable(true);
                  this.requestChange(request);
               
               }
           }
           
           if (_selectedCachedDataItem == null)
           {
               throw new IllegalActionException(" The selected entity has a null data (Maybe data download failed)");
           }
           // if no sql command, we need set cloumns
           if (!_hasSQLCommand)
           {
                 dbg.print("There is no sql command attribute and set up cloumns in compelete method", 2);
                  _columns = _selectedTableEntity.getFields();
           }
           reconfigurePorts(_columns);
       }      
    }
    
    /*
     * Method to find the entity index in EntityList which has the same name to the
     * given string. If no entity index found, -1 will be returned.
     */
    private int lookupEntityListName(String givenString)
    {
        int index = INDEXFORNOTFOUND;
        if (givenString != null && !givenString.trim().equals(""))
        {
          int size = _entityList.size();
          for (int i=0; i<size; i++)
          {
               Entity entity = (Entity)_entityList.elementAt(i);
               String entityName = entity.getName();
               if (entityName != null && !entityName.trim().equals("") && entityName.equals(givenString))
               {
                 index = i;
                 
               }
          }
        }
        dbg.print("The selected index is "+index, 2);
        return index;
    }
    
    /*
     * This method will generate selected cloumn list by given a 
     */
    private void generteLabelListAndTypteListFromColumns(Vector columns)
    {
        
        if (columns != null)
        {
          int size = columns.size();
          _selectedColumnLabelList = new String[size];
          _selectedColumnTypeList  = new Type[size];
          for(int i=0; i<size; i++)
          {
              DSTableFieldIFace column = (DSTableFieldIFace) columns.elementAt(i);
              _selectedColumnLabelList[i] = column.getName();
              String type = column.getDataType();
              _selectedColumnTypeList[i] = getBaseType(type);
                     
          }
        }
      
    }
    
    /*
     * Method to transform a string array to token array base on given type
     * 
     */
    private Token[] transformStringVectorToTokenArray(Vector stringVector, Type type)
                            throws IllegalActionException
    {
        if (stringVector == null)
        {
            return null;
        }
        int size = stringVector.size();
        Token[] columnToken    = new Token[size];
        //find the data type for each att
        if (type == BaseType.INT) 
        {
           for (int j =0; j<size; j++)
           {    
              String eleStr = (String)stringVector.elementAt(j); 
              dbg.print("The column value "+eleStr, 2);
              IntToken val = new IntToken(new Integer(eleStr).intValue());
              columnToken[j] = val;
           }
        } 
        else if (type == BaseType.DOUBLE) 
        {
           
            for (int j =0; j<size; j++)
            {    
               String eleStr = (String)stringVector.elementAt(j);  
               dbg.print("The column value "+eleStr, 2);
               DoubleToken val = new DoubleToken(new Double(eleStr).doubleValue());
               columnToken[j] = val;
            }
        } 
        else if (type == BaseType.LONG) 
        {  
           
            for (int j =0; j<size; j++)
            {    
               String eleStr = (String)stringVector.elementAt(j);  
               dbg.print("The column value "+eleStr, 2);
               LongToken val = new LongToken(new Long(eleStr).longValue());
               columnToken[j] = val;
            }
        }
        else if (type == BaseType.STRING) 
        {
            
            for (int j =0; j<size; j++)
            {    
               String eleStr = (String)stringVector.elementAt(j);
               dbg.print("The column value "+eleStr, 2);
               StringToken  val = new StringToken(eleStr);
               columnToken[j] = val;
            }
        } 
        else 
        {
            
            throw new IllegalActionException("Couldn't find token for this type");
        }
        return columnToken;
    }
}
