/**
 *    '$RCSfile: QuickSearchAction.java,v $'
 *
 *     '$Author: ruland $'
 *       '$Date: 2005/12/14 04:00:00 $'
 *   '$Revision: 1.33 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.ecogrid;


import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;

import org.ecoinformatics.ecogrid.queryservice.query.QueryType;
import org.ecoinformatics.ecogrid.queryservice.resultset.ResultsetType;
import org.ecoinformatics.seek.ecogrid.exception.InvalidEcogridQueryException;
import org.ecoinformatics.util.DebugPrinter;
import ptolemy.kernel.CompositeEntity;
import ptolemy.vergil.tree.EntityTreeModel;
import ptolemy.vergil.tree.VisibleTreeModel;


/**
 * This class will handle quick search action
 */

public class QuickSearchAction extends AbstractAction {
  // attributes
  private String searchValue = null;
  private String searchType = "All"; // Defaults to "All"
  private Vector searchServicesVector = null;
  private CompositeEntity resultRoot = null;
  private DebugPrinter dbg = null;
  // This vector will store a list of vector. One sub-vector is a list of
  //resultrecord got from one search scope after excuting query (probably search
  // mutiple endpoints)
  private Vector recordVector = null;
  private DatasetPanel datasetPanel = null;
  private EntityTreeModel treeModel = null;

  private Vector actionList = new Vector();
  private Vector recycledActionList = new Vector();
  private EcoGridQueryServicesController controller = null;
  private int totalResulsetItem = 0;

    /**
     * Construct of this class
     *
     * @param controller the search info includes endspoints, namespace etc
     * @param name the name of the action
     * @param datasetPane the datasetpane to fire the action
     */
    public QuickSearchAction(EcoGridQueryServicesController controller,
                           String name, DatasetPanel datasetPane) {
    super(name);
    dbg = DebugPrinter.getInstance();
    this.controller = controller;
    recordVector = new Vector();
    this.datasetPanel = datasetPane;

  }


  /**
   * Method to get the recordVector which store the ResultRecord after searching
   * @return Vector
   */
  public Vector getRecordVector() {
    return recordVector;
  }


  /**
   * Set up search scope vector
   * @param searchVector Vector
   */
  /*public void setSearchSerivcesVector(Vector searchVector)
     {
    searchServicesVector = searchVector;
     }*/


  /**
   * The todo Implementation of abstract method. It will search ecogrid site
   *
   * @param e ActionEvent
   */
  public void actionPerformed(ActionEvent e) {
    searchServicesVector = controller.getSelectedServicesList();
    actionList = new Vector();
    recycledActionList = new Vector();
    if (datasetPanel != null) {
      searchValue = datasetPanel.getSearchTextFieldValue();
      //searchType  = datasetPanel.getSearchDataSrcType();
      resultRoot = datasetPanel.getResultRoot();
    }

    //transfer endpoint based EcoGridService to namespace based Search Scope
    Vector searchScopeVector = transformEcoGridServiceToSearchScope();
    if (searchScopeVector != null && !searchScopeVector.isEmpty() && resultRoot != null
        && searchValue != null && !searchValue.trim().equals("")) {
      resultRoot.removeAllEntities();
      // go through every namespace in search scope
      for (int i = 0; i < searchScopeVector.size(); i++) {
        // vecotr to store the ResultRecord for one search scope
        Vector resultFromOneScope = new Vector();
        SearchScope searchScope = (SearchScope)searchScopeVector.elementAt(i);
        //String namespace = searchScope.getNamespace();
        // get quick search query from metadata specification class
        MetadataSpecificationInterface metadataSpecClass = searchScope.
          getMetadataSpecification();

        // *** Temporary Code
        String name = metadataSpecClass.getBriefName();
        String namespace = searchScope.getNamespace();
        if (!searchType.equals("All") && !searchType.equals(name)) {
          continue;
        }

        QueryType quickSearchQuery = null;
        try {
          quickSearchQuery = metadataSpecClass.getQuickSearchEcoGridQuery(
            searchValue);
        } catch (InvalidEcogridQueryException inE) {
          dbg.print("The error to generate quick search query " +
                    inE.getMessage(), 2);
          return;
        }
        Vector searchEndPoints = searchScope.getEndPoints();
        if (searchEndPoints == null) {
          dbg.print("No search end points can be found", 2);
          return;
        }

        // go through the end points vector and create query action

        searchEndPointsVector(resultFromOneScope, searchEndPoints,
                              quickSearchQuery, metadataSpecClass, namespace);

      } //for
      dbg.print("Inintiall query action ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ "
                + actionList.size(), 2);
      //start query action
      datasetPanel.startSearchProgressBar();
      for (int i = 0; i < actionList.size(); i++) {
        QueryAction queryAction = (QueryAction)actionList.elementAt(i);
        queryAction.actionPerformed(null);
      }
    } //if
  } //actionPerformed


  /*cd
   * Go through the search end vector
   */
  private void searchEndPointsVector(Vector resultFromOneScope,
                                     Vector searchEndPoints,
                                     QueryType quickSearchQuery,
                                     MetadataSpecificationInterface
                                     metadataSpecClass,
                                     String namespace) {

    // go through each search end points and get result set
    for (int j = 0; j < searchEndPoints.size(); j++) {

      String url = (String)searchEndPoints.elementAt(j);
      dbg.print("============search " + url + " for namespace " + namespace, 2);
      QueryAction doQuery = new QueryAction(this, quickSearchQuery, url,
                                            searchValue,
                                            datasetPanel, metadataSpecClass,
                                            namespace);
      actionList.addElement(doQuery);

    } //for
    //actionList = new Vector();

  }


  /*
   * Method transfer EcoGridService(base on endpoint) vector to SearchScope
   * (base on namespace) vector
   */
  public Vector transformEcoGridServiceToSearchScope() {
    Vector searchScopeVector = new Vector();
    if (searchServicesVector != null) {
      int size = searchServicesVector.size();
      //this hashtable will stored the namespace and its searchscope
      Hashtable namespaceAndSearhScopeMap = new Hashtable();
      for (int i = 0; i < size; i++) {
        SelectableEcoGridService service = (SelectableEcoGridService)
                                           searchServicesVector.elementAt(i);
        if (service != null) {
          String endPoint = service.getEndPoint();
          DocumentType[] typeList = service.getSelectedDocumentTypeList();
          if (endPoint != null && !endPoint.trim().equals("") && typeList != null) {
            //go through every type
            for (int j = 0; j < typeList.length; j++) {
              DocumentType type = typeList[j];
              String namespaceStr = type.getNamespace();
              String metadataClassName = type.getMetadataSpecificationClassName();
              if (namespaceStr != null && !namespaceStr.trim().equals("") &&
                  metadataClassName != null
                  && !metadataClassName.trim().equals("")) {
                if (!namespaceAndSearhScopeMap.containsKey(namespaceStr)) {
                  //if this is first namespace we found, create a new searchscope
                  Vector endPointsVector = new Vector();
                  endPointsVector.add(endPoint);
                  try {
                    SearchScope newScope = new SearchScope(namespaceStr,
                                                           metadataClassName,
                                                           endPointsVector);
                    namespaceAndSearhScopeMap.put(namespaceStr, newScope);
                    dbg.print("add " + namespaceStr + " " +
                              metadataClassName + " " + endPoint +
                              " into SearchScope", 2);
                  } catch (Exception e) {
                    dbg.print("error create search scope " + e.getMessage(), 2);
                    continue;
                  }
                } else {
                  //if we already has a SearchScope for this namespace,
                  // add this endpoints to endpoints vector
                  SearchScope existedScope = (SearchScope)
                                             namespaceAndSearhScopeMap.get(
                    namespaceStr);
                  dbg.print("Add endpoint " + endPoint
                            + " to a existed searchscope object", 2);
                  existedScope.addSearchEndPoint(endPoint);
                  //overwrite the old value
                  namespaceAndSearhScopeMap.put(namespaceStr, existedScope);
                }
              } //if
            } //for
          } //if
        } //if
      } //for

      //transfer the hashtable to a vector
      Enumeration en = namespaceAndSearhScopeMap.keys();
      while (en.hasMoreElements()) {
        String key = (String)en.nextElement();
        SearchScope scope = (SearchScope)namespaceAndSearhScopeMap.get(key);
        if (scope != null) {
          searchScopeVector.add(scope);
        }
      }

    } //if

    return searchScopeVector;
  }


  /**
   * Method to stop quick search.
   *
   */
  public void stop() {

    //then will should stop the queryaction(they are in another thread)
    int size = actionList.size();
    dbg.print("The size of original query action in stop method is " + size, 2);
    for (int i = 0; i < size; i++) {
      QueryAction queryAct = (QueryAction)actionList.elementAt(i);
      synchronized (recycledActionList) {
        if (!recycledActionList.contains(queryAct)) {
          queryAct.stopAction();
        }
      }
    }
    datasetPanel.resetResultsPanel();
    datasetPanel.setProgressLabel("search canceled.");
    //reset total result set item
    totalResulsetItem = 0;
    actionList = new Vector();
    recycledActionList = new Vector();
    datasetPanel.resetSearchPanel();
  }


  public synchronized void recycleQueryAction(QueryAction queryAction) {
    // old canceled action wouldn't add to recycle vector
    if (!queryAction.isCanceled() && actionList.contains(queryAction)) {
      dbg.print("recyled a query action !!!!!!!!!!!!!" +
                queryAction.getNameSpace() + " " + queryAction.getSearchValue(),
                2);
      recycledActionList.add(queryAction);
    } else {
      dbg.print("descard an canceled query action !!!!!!!!!!!!!" +
                queryAction.getNameSpace() + " " + queryAction.getSearchValue(),
                2);
    }
    // after recycled every query action, display the panel
    int sizeOfRecycle = recycledActionList.size();
    dbg.print("The recyleed action size==== " + sizeOfRecycle, 2);
    int sizeOfOrigin = actionList.size();
    dbg.print("The origanl action size ==== " + sizeOfOrigin, 2);
    if (!recycledActionList.isEmpty() && sizeOfRecycle == sizeOfOrigin) {
      dbg.print("finalize the resultset.........", 2);
      for (int i = 0; i < sizeOfOrigin; i++) {
        QueryAction action = (QueryAction)recycledActionList.elementAt(i);
        resultsetComplete(action);
      }
    }
  }


    /**
     * Processes the resultset and fills the panel
     *
     * @param aQA QueryAction
     */
    private void resultsetComplete(QueryAction aQA) {
    // parse the resultset into and stored it into a vector
    Vector resultVector = new Vector();
    int numResults = 0;
    try {

      MetadataSpecificationInterface mdi = aQA.getMetaDataIFace();
      ResultsetType results = aQA.getResultSet();

      //_queryAction = aQA;
      if (results != null) {
        mdi.addResultsetRecordsToContainer(results, aQA.getURL(), resultRoot,
                                           resultVector);
        int newHits = mdi.getNumResults();
        // Note: this must be done after calling addResultsetRecordsToContainer
        // addResultsetRecordsToContainer does the processing
        totalResulsetItem = totalResulsetItem + newHits;
      }
      /*else
                 {
          // using SwingUtilities lets the msg dialog (model) show on it's own thread instead of
          // blocking here and keeping the progress meter from ending
          SwingUtilities.invokeLater(new Runnable() {
              public void run() {
       JOptionPane.showMessageDialog(datasetPanel, _queryAction.getError());
              }
            });
          datasetPanel.setProgressLabel("Error getting results.");
                 }*/

    } catch (Exception ee) {
      dbg.print("Error adding resultset to container " + ee.getMessage(), 2);
    }

    actionList.removeElement(aQA);

    if (resultVector.size() > 0) {
      recordVector.add(resultVector);
    }

    if (actionList.size() == 0) {
      dbg.print("update dataset panel", 2);
      // sort the resultvector from one scope and add it to the recordVector
      //Collections.sort(resultFromOneScope, new ResultRecordComparator());
      treeModel = new VisibleTreeModel(resultRoot);
      datasetPanel.update(treeModel);
      datasetPanel.setProgressLabel(totalResulsetItem + " results returned.");
      datasetPanel.resetSearchPanel();
      //reset total result set item
      totalResulsetItem = 0;
    }

  }
}
