/*
 Copyright (c) 1997-2003 The Regents of the University of California.
 All rights reserved.
 Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.

 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

                                        PT_COPYRIGHT_VERSION_2
                                        COPYRIGHTENDKEY

 @ProposedRating
 @AcceptedRating

 */

package org.kepler.gui;


import java.util.Iterator;

import ptolemy.kernel.util.Attribute;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.NamedObj;


/**
 * This class allows the configuration option keplerSpecificMenusDialogs to
 * be passed via the moml configuration.  The MenusDialogsModifier class
 * contains a flag which determines if the kepler-specific menus and dialogs
 * should be displayed, and Ptolemy's menus and dialogs suppressed
 */
public class MenusAndDialogsModifierFactory extends Attribute {

  /** Create a factory with the given name and container.
   *  @param container The container.
   *  @param name The name.
   *  @exception IllegalActionException If the container is incompatible
   *   with this attribute.
   *  @exception NameDuplicationException If the name coincides with
   *   an attribute already in the container.
   */
  public MenusAndDialogsModifierFactory(NamedObj container, String name) throws
    IllegalActionException, NameDuplicationException {
    super(container, name);
  }


  ///////////////////////////////////////////////////////////////////
  ////                         public methods                    ////
  ///////////////////////////////////////////////////////////////////

    /**
     * creates the MenusDialogsModifier class and returns it
     *
     * @return MenusDialogsModifier
     */
    public MenusDialogsModifier createMenusDialogsModifier() {
    MenusDialogsModifier modifier = null;
    Iterator factories
      = attributeList(MenusAndDialogsModifierFactory.class).iterator();
    while (factories.hasNext() && modifier == null) {
      MenusAndDialogsModifierFactory factory =
        (MenusAndDialogsModifierFactory)factories.next();
      modifier = factory.createMenusDialogsModifier();
    }
    return modifier;
  }
}
