/**
 *    '$RCSfile: DataCacheManager.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2005/12/16 22:19:15 $'
 *   '$Revision: 1.44 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.kepler.objectmanager.cache;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

import org.ecoinformatics.seek.datasource.EcogridQueryDataCacheItem;
import org.ecoinformatics.seek.querybuilder.DBUIUtils;
import org.ecoinformatics.util.DebugPrinter;
import org.ecoinformatics.util.Config;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * A Singleton that managers DataCaheItems
 */
public class DataCacheManager implements Serializable
{
  // static data member
  private static String mCacheFileName;
  private static DataCacheManager mGlobalCacheManager = null;
  private static DebugPrinter dbg = DebugPrinter.getInstance();
  private final static String CLASSPATH = System.getProperty("KEPLER")
                        + File.separator + "build" + File.separator +
                        "classes" + File.separator;
  // non-static members
  /**
   *  Description of the Field
   */
  protected Vector mCache = new Vector();
  /**
   *  Description of the Field
   */
  protected Vector mDeletedItems = new Vector();
  /**
   *  Description of the Field
   */
  protected boolean mDoingSave = false;

  /**
   * Default Constructor
   */
  protected DataCacheManager()
  {
    mCacheFileName = prepareCacheLocation();
    loadCache();
  }

  //-------------------------------------------------------------------
  // STATIC METHODS
  //-------------------------------------------------------------------

  /**
   * Returns the singleton
   *
   *@return
   */
  public static DataCacheManager getInstance()
  {
    if(mGlobalCacheManager == null)
    {
      mCacheFileName = "";
      mGlobalCacheManager = new DataCacheManager();
    }
    return mGlobalCacheManager;
  }

  /**
   * Look up an item in the cache, if it is not there than it creates one
   *
   *@param  aListener      the listener to attach
   *@param  aName          the name
   *@param  aResourceName  the resource name
   *@param  aClassName     the class name of the new that is to be created if it is not found
   *@return                the cache item
   */
  public static DataCacheObject getCacheItem(DataCacheListener aListener,
      String aName,
      String aResourceName,
      String aClassName)
  {
    return getInstance().getCacheItemInternal(aListener, aName, aResourceName, aClassName, false);
  }

  /**
   * Look up an item in the cache, if it is not there than it creates one
   *
   *@param  aListener           the listener to attach
   *@param  aName               the name
   *@param  aResourceName       the resource name
   *@param  aClassName          the class name of the new that is to be created if it is not found
   *@param  aIgnoreResOnSearch  search only by Name
   *@return                     the cache item
   */
  public static DataCacheObject getCacheItem(DataCacheListener aListener,
      String aName,
      String aResourceName,
      String aClassName,
      boolean aIgnoreResOnSearch)
  {
    return getInstance().getCacheItemInternal(aListener, aName, aResourceName, aClassName, aIgnoreResOnSearch);
  }

  /**
   * Gets a File by Name and Resouce Name
   *
   *@param  aName               the name
   *@param  aResource           the file type
   *@return                     the cache item
   */
  public static DataCacheFileObject getFile(String aName, String aResource)
  {
    return (DataCacheFileObject)getInstance().lookupCacheItemInternal(aName, aResource, DataCacheFileObject.class.getName(), false);
  }

  /**
   * Gets a File by name only
   *
   *@param  aLogicalName  the name
   *@return               the cache item
   */
  public static DataCacheFileObject getFile(String aLogicalName)
  {
    return (DataCacheFileObject)getInstance().lookupCacheItemInternal(aLogicalName, "", DataCacheFileObject.class.getName(), true);
  }

  /**
   * Gets a File by name and named/value pair
   *
   *@param  aLogicalName  the name
   *@param  aAttr
   *@param  aValue
   *@return               the cache item
   */
  public static DataCacheFileObject getFile(String aLogicalName, String aAttr, String aValue)
  {
    return (DataCacheFileObject)getInstance().lookupByNameAndAttrInternal(aLogicalName, aAttr, aValue);
  }

  /**
   * Gets a cache item by name and attr value
   *
   *@param  aName   the name
   *@param  aAttr   the name of the attribute
   *@param  aValue  the value of the named attr
   *@return         the cache item
   */
  public static DataCacheObject getItem(String aName, String aAttr, String aValue)
  {
    return (DataCacheObject)getInstance().lookupByNameAndAttrInternal(aName, aAttr, aValue);
  }

  /**
   * Returns a list of cache item with the same attr value
   *
   *@param  aAttr   name of attr
   *@param  aValue  value of attr
   *@return         the cache item
   */
  public static Vector getCacheItemsByAttr(String aAttr, String aValue)
  {
    return getInstance().getCacheItemsByAttrInternal(aAttr, aValue);
  }

  /**
   * Returns a list of all cache items
   *
   * @return         the cache item list
   */
  public static Enumeration getCacheItems()
  {
    return getInstance().getCacheItemsInternal();
  }

  /**
   * Return an item fro the cache, but does not create it
   *
   *@param  aName               the name
   *@param  aResourceName       the resource name
   *@param  aClassName          the class name of the new that is to be created if it is not found
   *@param  aIgnoreResOnSearch  search only by Name
   *@return                     the cache item
   */
  public static DataCacheObject lookupCacheItem(String aName,
      String aResourceName,
      String aClassName,
      boolean aIgnoreResOnSearch)
  {
    return getInstance().lookupCacheItemInternal(aName, aResourceName, aClassName, aIgnoreResOnSearch);
  }

  /**
   * This method will add a file to the cache. You can create a new empty cache file
   * and load the data in via a byte array, or by "tranferring" an existing external
   * file "into" the cache, or by enabling a cache entry to point to an external file.
   *
   * The file then be saved into the cache as follows:
   * *
   * INTERNAL - invalid value for aFileType and will the return value will always be null
   *
   * EXTERNAL - The cache name of the file will be "aName" and the resource name
   *            will be set to the true external file name.
   *
   * COPY_FILE_TO_CACHE - The will "move" the external file into the cache, but will NOT
   *                      delete the external file.
   *
   *@param  aPhysicalFileName  the "physical" name of a file (this name will not be stored if aFileLocation is COPY_FILE_TO_CACHE)
   *@param  aLogicalName       the "logical" name of a file unrelated to the physical name
   *@param  aType              a description of the file's type (optional and can be null)
   *@param  aFileLocation      the file location type (see DataCacheFileObject for values)
   *@return                    the cache item
   */
  public static DataCacheFileObject putFile(String aPhysicalFileName,
    String aLogicalName, String aType, int aFileLocation)
  {
    if(aFileLocation != DataCacheFileObject.INTERNAL)
    {
      return getInstance().createFileItemInternal(aPhysicalFileName,
        aLogicalName, aType, aFileLocation);
    }
    return null;
  }

  /**
   * Creates an empty file cache item and the byte array (data) can be set into
   * it and saved. Implies it is an internal file.
   *
   *@param  aLogicalName  the "logical" name of a file unrelated to the physical name
   *@param  aType         a description of the file's type (optional and can be null)
   *@return               the cache item
   */
  public static DataCacheFileObject putFile(String aLogicalName, String aType)
  {
    return getInstance().createFileItemInternal(null, aLogicalName, aType,
      DataCacheFileObject.INTERNAL);
  }

  /**
   * Creates an empty file cache item and the byte array (data) can be set into it and saved. Implies it is an internal file.
   *
   *@param  aLogicalName  the "logical" name of a file unrelated to the physical name
   *@param  aType         a description of the file's type (optional and can be null)
   *@return               the cache item
   */
  public static DataCacheFileObject putFile(String aLogicalName,
    InputStream aFileInputStream) throws IOException
  {
    return getInstance().createFileItemInternal(aLogicalName,
      aFileInputStream);
  }

  /**
   * This method adds an inputstream with a logical name and type into the cache.
   *
   *@param  aLogicalName       the "logical" name of a file unrelated to the physical name
   *@param  aType              a description of the file's type (optional and can be null)
   *@param  aFileInputStream   The stream to init the cache item with
   *@return                    the cache item
   */
  public static DataCacheFileObject putFile(String aLogicalName, String aType,
    InputStream aFileInputStream) throws IOException
  {
    return getInstance().createFileItemInternal(aLogicalName, aType,
      aFileInputStream);
  }

  /**
   * Creates a cache file with bytes and returns a file cache item
   *
   *@param  aLogicalName  the name
   *@param  aFileType     the file type
   *@param  aData
   *@return               the cache item
   */
  public static DataCacheFileObject createFileItemWithBytes(String aLogicalName, String aFileType, byte[] aData)
  {
    return getInstance().createFileItemWithBytesInternal(aLogicalName, aFileType, aData);
  }

  /**
   * Returns the length of the cache
   *
   *@return
   */
  public int getSize()
  {
    return mCache.size();
  }

  /**
   * Returns all the items in the cache
   *
   *@return
   */
  public Vector getItems()
  {
    return mCache;
  }

  /**
   * Returns the data path to the cache
   *
   *@return
   */
  public String getDataPath()
  {
    if(mCacheFileName.length() == 0)
    {

    }
    return Config.getCacheDirPath() + "cachedata" + File.separator;
  }

  /**
   * Retrieves a single item
   *
   *@param  aInx  the index of the item to be gotten
   *@return       The itemAt value
   */
  public DataCacheObject getItemAt(int aInx)
  {
    if(aInx >= 0 && aInx < mCache.size())
    {
      return (DataCacheObject)mCache.elementAt(aInx);
    }
    return null;
  }

  /**
   * return a Vector of all of the cache items.
   */
  public Enumeration getCacheItemsInternal()
  {
    return mCache.elements();
  }

  /**
   * Returns a list of cache item with the same attr value
   *
   *@param  aAttr   name of attr
   *@param  aValue  value of attr
   *@return         the list of items matching that criteria
   */
  public Vector getCacheItemsByAttrInternal(String aAttr, String aValue)
  {
    Vector items = null;

    for(Enumeration e = mCache.elements(); e.hasMoreElements(); )
    {
      DataCacheObject item = (DataCacheObject)e.nextElement();
      String value = item.getAttr(aAttr);

      if(value != null && value.equals(aValue))
      {
        if(items == null)
        {
          items = new Vector();
        }
        items.addElement(item);
      }
    }
    return items;
  }

  /**
   * Loads cache from disk
   *
   *@return
   */
  public boolean loadCache()
  {
    Document dom = DBUIUtils.readXMLFile2DOM(mCacheFileName);

    if(dom != null)
    {
      Node cache = DBUIUtils.findNode(dom, "cache");

      if(cache != null)
      {
        NodeList list = cache.getChildNodes();

        if(list != null)
        {
          for(int i = 0; i < list.getLength(); i++)
          {
            Node child = list.item(i);

            if(child.getNodeType() != Node.TEXT_NODE)
            {
              String className = DBUIUtils.findAttrValue(child, "classname");

              try
              {
                DataCacheObject item = getNewItemFromClassName(className);

                if(item != null)
                {
                  item.buildFromDOM(child);
                  addItem(item);
                }
              }
              catch(Exception e)
              {
                System.err.println(e);
              }
            }
          }
        }
      }
    }
    return false;
  }

  /**
   * Saves all the items out to disk
   *
   *@return    true on success
   */
  public boolean saveCache()
  {
    while(mDoingSave)
    {
      //System.err.println("waiting for saving cache");
    }
    mDoingSave = true;
    try
    {
      // This must be done first, it fills in the mLocalFileName
      for(Enumeration e = mCache.elements(); e.hasMoreElements(); )
      {
        DataCacheObject item = (DataCacheObject)e.nextElement();

        item.saveData();
      }

      try
      {
        BufferedWriter bufWriter = new BufferedWriter(new FileWriter(mCacheFileName));

        bufWriter.write(toXMLString());
        bufWriter.close();
      }
      catch(Exception e)
      {
        System.err.println("saveCache - " + e);
        return false;
      }

      // This is done last
      for(Enumeration e = mDeletedItems.elements(); e.hasMoreElements(); )
      {
        DataCacheObject item = (DataCacheObject)e.nextElement();

        item.clear();
      }
      mDeletedItems.clear();
    }
    catch(Exception e)
    {
      System.err.println(e);
    }
    mDoingSave = false;
    return true;
  }

  /**
   * Removes item from cache, and places it in a deleted items list.
   * This way if the cache is not saved then the items are not actually deleted.
   *
   *@param  aItem  to be removed
   */
  public void removeItem(DataCacheObject aItem)
  {
    mDeletedItems.addElement(aItem);
    aItem.removeAllListeners();
    mCache.removeElement(aItem);
  }

  /**
   * Removes item from cache, and places it in a deleted items list.
   * This way if the cache is not saved then the items are not actually deleted.
   *
   *@param  aInx  the index of the item to be removed
   */
  public void removeItem(int aInx)
  {
    if(aInx > 0 && aInx < mCache.size())
    {
      DataCacheObject item = (DataCacheObject)mCache.elementAt(aInx);

      mDeletedItems.addElement(item);
      item.removeAllListeners();
      mCache.removeElementAt(aInx);
    }
  }

  /**
   * Removes all the objects from the cache identified by an array of indexes
   *
   *@param  aIndexes  array of indexes to delete
   */
  public void removeItems(int[] aIndexes)
  {
    // Start by collecting all the objects to be deleted
    Vector delItems = new Vector();

    for(int i = 0; i < aIndexes.length; i++)
    {
      delItems.addElement(mCache.elementAt(aIndexes[i]));
    }

    // then remove them from the main cache and also add then to the deleted list.
    for(Enumeration e = delItems.elements(); e.hasMoreElements(); )
    {
      DataCacheObject item = (DataCacheObject)e.nextElement();

      mCache.remove(item);
      mDeletedItems.addElement(item);
    }
    delItems.clear();
    delItems = null;
  }

  /**
   * Clear the cache and removes all items
   */
  public void clear()
  {
    for(Enumeration e = mCache.elements(); e.hasMoreElements(); )
    {
      DataCacheObject item = (DataCacheObject)e.nextElement();

      item.clear();
    }
    mCache.clear();

    // Clear the deleted list also
    for(Enumeration e = mDeletedItems.elements(); e.hasMoreElements(); )
    {
      DataCacheObject item = (DataCacheObject)e.nextElement();

      item.clear();
    }
    mDeletedItems.clear();
    saveCache();
  }

  /**
   * Clear the cache and removes all items
   *
   *@param  aName
   */
  public void clearAllByName(String aName)
  {
    Vector itemsToRemove = new Vector();

    for(Enumeration e = mCache.elements(); e.hasMoreElements(); )
    {
      DataCacheObject item = (DataCacheObject)e.nextElement();

      if(item.getName().equals(aName))
      {
        itemsToRemove.addElement(item);
      }
    }

    for(Enumeration e = itemsToRemove.elements(); e.hasMoreElements(); )
    {
      DataCacheObject item = (DataCacheObject)e.nextElement();

      item.clear();
      mCache.remove(item);
    }
  }

  /**
   * Looks up the File item and if it isn't there than it creates one
   *
   *@param  aLogicalName       the "logical" name of a file unrelated to the physical name
   *@param  aType              a description of the file's type (optional and can be null)
   *@param  aFileLocation      the file location type (see DataCacheFileObject for values)
   *@param  aPhysicalName
   *@return                    a cache file item
   */
  public DataCacheFileObject createFileItemInternal(String aPhysicalName, String aLogicalName, String aType, int aFileLocation)
  {
    DataCacheObject item = lookupCacheItemInternal(aLogicalName, aType, DataCacheFileObject.class.getName(), false);
    DataCacheFileObject fileItem = item != null ? (DataCacheFileObject)item : new DataCacheFileObject();

    fileItem.initializeWithFileName(aPhysicalName, aLogicalName, aType, aFileLocation);
    if(item == null)
    {
      mCache.addElement(fileItem);
      saveCache();
    }
    return fileItem;
  }

  /**
   * Looks up the File item and if it isn't there than it creates one
   *
   *@param  aLogicalName       the "logical" name of a file unrelated to the physical name
   *@param  aType              a description of the file's type (optional and can be null)
   *@param  aFileLocation      the file location type (see DataCacheFileObject for values)
   *@param  aPhysicalName
   *@return                    a cache file item
   */
  public DataCacheFileObject createFileItemInternal(String aLogicalName,
    InputStream aFileInputStream) throws IOException
  {
    DataCacheObject item = lookupCacheItemInternal(aLogicalName,
      null, DataCacheFileObject.class.getName(), false);
    DataCacheFileObject fileItem = item != null ? (DataCacheFileObject)item :
      new DataCacheFileObject();

    fileItem.initializeWithInputStream(aLogicalName,
      aFileInputStream);
    if(item == null)
    {
      mCache.addElement(fileItem);
      saveCache();
    }
    return fileItem;
  }

  /**
   * Looks up the File item and if it isn't there than it creates one
   *
   *@param  aLogicalName       the "logical" name of a file unrelated to the physical name
   *@param  aType              a description of the file's type (optional and can be null)
   *@param  aFileLocation      the file location type (see DataCacheFileObject for values)
   *@param  aPhysicalName
   *@return                    a cache file item
   */
  public DataCacheFileObject createFileItemInternal(String aLogicalName,
    String aType, InputStream aFileInputStream) throws IOException
  {
    DataCacheObject item = lookupCacheItemInternal(aLogicalName, aType,
      DataCacheFileObject.class.getName(), false);
    DataCacheFileObject fileItem = item != null ? (DataCacheFileObject)item : new DataCacheFileObject();

    fileItem.initializeWithInputStream(aLogicalName, aType, aFileInputStream);
    if(item == null)
    {
      mCache.addElement(fileItem);
      saveCache();
    }

    return fileItem;
  }

  /**
   * Creates a cache file with bytes and returns a file cache item,
   * the byte arary is written into the physical cache file
   *
   *@param  aLogicalName  the name
   *@param  aType         the file type
   *@param  aData
   *@return               the new item
   */
  public DataCacheFileObject createFileItemWithBytesInternal(String aLogicalName, String aType, byte[] aData)
  {
    DataCacheObject item = lookupCacheItemInternal(aLogicalName, null, DataCacheFileObject.class.getName(), true);
    DataCacheFileObject fileItem = item != null ? (DataCacheFileObject)item : new DataCacheFileObject();

    fileItem.initializeWithBytes(aLogicalName, aType, aData);
    if(item == null)
    {
      mCache.addElement(fileItem);
    }
    return fileItem;
  }

  /**
   * Gets a File by name and named/value pair.
   * Meaning it must have an attribute of the requested name and the value must match
   *
   *@param  aName   the name
   *@param  aAttr   the name of the attribute
   *@param  aValue  the value of the named attr
   *@return         the item
   */
  public DataCacheObject lookupByNameAndAttrInternal(String aName, String aAttr, String aValue)
  {
    for(Enumeration e = mCache.elements(); e.hasMoreElements(); )
    {
      DataCacheObject item = (DataCacheObject)e.nextElement();
      String value = item.getAttr(aAttr);

      if(value != null && value.equals(aValue))
      {
        return item;
      }
    }
    return null;
  }

  /**
   * Creates an XML string that describes the cache (this is what is saved out to disk)
   *
   *@return
   */
  public String toXMLString()
  {
    StringBuffer strBuf = new StringBuffer();

    strBuf.append("<cache>\n");
    for(Enumeration e = mCache.elements(); e.hasMoreElements(); )
    {
      DataCacheObject item = (DataCacheObject)e.nextElement();

      strBuf.append(item.toXMLString());
    }
    strBuf.append("</cache>\n");
    return strBuf.toString();
  }

  /**
   * Look up an item in the cache, if it is not there than it creates one
   *
   *@param  aListener      the listener to attach
   *@param  aName          the name
   *@param  aResourceName  the resource name
   *@param  aClassName     the class name of the new that is to be created if it is not found
   *@return                the item
   */
  private DataCacheObject getCacheItemInternal(DataCacheListener aListener,
      String aName,
      String aResourceName,
      String aClassName)
  {
    return getCacheItemInternal(aListener, aName, aResourceName, aClassName, false);
  }

  /**
   * Look up an item in the cache, if it is not there than it creates one using the class name
   *
   *@param  aName               the name
   *@param  aResourceName       the resource name
   *@param  aClassName          the class name of the new that is to be created if it is not found
   *@param  aIgnoreResOnSearch  search only by Name
   *@param  aListener
   *@return                     the item
   */
  private DataCacheObject getCacheItemInternal(DataCacheListener aListener,
      String aName,
      String aResourceName,
      String aClassName,
      boolean aIgnoreResOnSearch)
  {
    int maxWaitTime = 100;
    //System.err.println("**** Looking Name["+aName+"] Res["+aResourceName+"] Class["+aClassName+"]");
    DataCacheObject item = lookupCacheItemInternal(aName, aResourceName, aClassName, aIgnoreResOnSearch);
    
    if(item != null)
    {
      //System.out.println("the  calss name is!!!!!!!!!!!!!!!!!!!! "+item.getSerialClassName());
      if (item.getSerialClassName() != EcogridQueryDataCacheItem.class.getName())
      {
         int index = 0;
         while ( index < maxWaitTime && item.hasListener())
         {
             //System.out.println("=========================before wait");
             try
             {
               Thread.sleep(10000);
               
             }
             catch (Exception e)
             {
               
             }
             index++;
             //System.out.println("=============================after wait");
         }
      }
      item.removeAllListeners();
      item.addListener(aListener);
      return item;
    }
    //System.err.println("**** Creating ["+aName+"]["+aResourceName+"]["+aClassName+"]");
    item = getNewItemFromClassName(aClassName);
    if(item != null)
    {
      item.setName(aName);
      item.setResourceName(aResourceName);
      item.addListener(aListener);
      mCache.addElement(item);
    }

    return item;
  }

  /**
   * Return an item from the cache, but does not create it
   *
   *@param  aName               the name of the item
   *@param  aResourceName       the resource name
   *@param  aClassName          the class name of the new that is to be created if it is not found
   *@param  aIgnoreResOnSearch  search only by Name and class, but not by resource name
   *@return
   */
  private DataCacheObject lookupCacheItemInternal(String aName,
      String aResourceName,
      String aClassName,
      boolean aIgnoreResOnSearch)
  {
    boolean found = false;

    for(Enumeration e = mCache.elements(); e.hasMoreElements(); )
    {
      DataCacheObject item = (DataCacheObject)e.nextElement();

      //System.out.println("itemName["+item.getName()+"] aName["+aName+"]");
      if(item.getName().equals(aName) && item.getSerialClassName().equals(aClassName))
      {
        boolean resNameOK = true;

        if(!aIgnoreResOnSearch)
        {
          String resName = item.getResourceName();

          if(resName != null && aResourceName != null)
          {
            //System.out.println("resName["+resName+"] aResourceName["+aResourceName+"]");
            resNameOK = resName.equals(aResourceName);
          }
          else
          {
            resNameOK = false;
          }
        }

        if(resNameOK)
        {
          return item;
        }
      }
    }
    return null;
  }

  /**
   * CReates a new object of the class name
   *
   *@param  aClassName  name of class to be created
   *@return             the object
   */
  protected DataCacheObject getNewItemFromClassName(String aClassName)
  {
    try
    {
      Class classObj = Class.forName(aClassName);

      return (DataCacheObject)classObj.newInstance();
    }
    catch(Exception e)
    {
      System.err.println(e);
    }

    return null;
  }

  /**
   *  Description of the Method
   */
  protected void finalize()
  {
  }

  /**
   * Creates the directories for storing the cache (if necessary)
   *
   *@return    the path to the cache
   */
  protected String prepareCacheLocation()
  {
    if(mCacheFileName.length() == 0)
    {
      StringTokenizer st = new StringTokenizer(Config.getCacheDirPath(), File.separator);
      StringBuffer dirPath = new StringBuffer();

      while(st.hasMoreTokens())
      {
        String dirName = st.nextToken();

        if(dirName.length() > 0)
        {
          if(dirPath.length() >= 0)
          {
            dirPath.append(File.separator);
          }
          dirPath.append(dirName);

          File file = new File(dirPath.toString());

          if(!file.exists())
          {
            file.mkdir();
          }
        }
      }
    }
    return Config.getCacheDirPath() + "cachedata" + File.separator +  "data_cache.dat";
  }

  /**
   * Adds item to cache vector
   *
   *@param  aItem
   */
  protected void addItem(DataCacheObject aItem)
  {
    mCache.addElement(aItem);
  }
}

