/**
 *  '$RCSfile: EML2MetadataSpecificationTest.java,v $'
 *  Copyright: 2000 Regents of the University of California and the
 *              National Center for Ecological Analysis and Synthesis
 *    Authors: @authors@
 *    Release: @release@
 *
 *   '$Author: ruland $'
 *     '$Date: 2005/12/15 19:29:36 $'
 * '$Revision: 1.8 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package test.org.ecoinformatics.seek.ecogrid;

import edu.ucsb.nceas.configxml.ConfigXML;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;


import org.ecoinformatics.ecogrid.queryservice.query.ConditionType;
import org.ecoinformatics.ecogrid.queryservice.query.OperatorType;
import org.ecoinformatics.ecogrid.queryservice.query.ANDType;
import org.ecoinformatics.ecogrid.queryservice.query.QueryType;
import org.ecoinformatics.ecogrid.queryservice.query.QueryTypeNamespace;
import org.ecoinformatics.ecogrid.queryservice.resultset.ResultsetType;
import org.ecoinformatics.seek.ecogrid.EML2MetadataSpecification;
import org.ecoinformatics.seek.ecogrid.ResultRecord;
import org.ecoinformatics.seek.ecogrid.ResultRecordDetail;

import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.Attribute;
import ptolemy.kernel.util.StringAttribute;
import java.util.Enumeration;
import java.util.Date;
import java.util.TimeZone;
import java.util.Calendar;
import org.ecoinformatics.ecogrid.queryservice.resultset.ResultsetTypeResultsetMetadata;
import org.ecoinformatics.ecogrid.EcogridUtils;
import org.ecoinformatics.ecogrid.queryservice.resultset.ResultsetTypeRecord;
import org.ecoinformatics.ecogrid.queryservice.resultset.ResultsetTypeResultsetMetadataRecordStructure;
import org.ecoinformatics.ecogrid.queryservice.resultset.ResultsetTypeResultsetMetadataRecordStructureReturnField;
import org.apache.axis.types.URI;
import org.ecoinformatics.ecogrid.queryservice.resultset.ResultsetTypeRecordReturnField;
import org.ecoinformatics.ecogrid.queryservice.resultset.ResultsetTypeResultsetMetadataNamespace;



/**
 * A JUnit test for testing Step class processing
 */
public class EML2MetadataSpecificationTest extends TestCase
{
  ConfigXML config;
  EML2MetadataSpecification eml2Metadata;

  private static final String SEARCHVALUE = "value";
  private static final String ID1         = "foo.1.1";
  private static final String ID2         = "foo.2.1";
  private static final String TITLE1      = "soil1";
  private static final String TITLE2      = "soil2";
  private static final String ENTITY11    = "entity11";
  private static final String ENTITY12    = "entity12";
  private static final String ENTITY21    = "entity21";
  private static final String ENTITY22    = "entity22";
  private static final String ENDPOINT    = "http://dev.nceas.ucsb.edu";

  /**
   * Constructor to build the test
   *
   * @param name the name of the test method
   */
  public EML2MetadataSpecificationTest(String name)
  {
    super(name);
    eml2Metadata = new EML2MetadataSpecification();

  }

  /**
   * Establish a testing framework by initializing appropriate objects
   */
  public void setUp()
  {
  }

  /**
   * Release any objects after tests are complete
   */
  public void tearDown()
  {
  }

  /**
   * Create a suite of tests to be run together
   */
  public static Test suite()
  {
    TestSuite suite = new TestSuite();
    suite.addTest(new EML2MetadataSpecificationTest("initialize"));
    suite.addTest(
         new EML2MetadataSpecificationTest("getQuickSearchEcoGridQueryTest"));
    return suite;
  }

  /**
   * Run an initial test that always passes to check that the test
   * harness is working.
   */
  public void initialize()
  {
    assertTrue(1 == 1);
  }

  /**
   * The test for method getQuickSearchEcoGridQuery. This method will generate
   * a ecogrid query for quick search
   */
  public void getQuickSearchEcoGridQueryTest()
  {
    try
    {
       QueryType query = eml2Metadata.getQuickSearchEcoGridQuery(SEARCHVALUE);

       String str = org.ecoinformatics.ecogrid.queryservice.util.EcogridQueryTransformer.toXMLString( query );
       System.err.println("Query is: ");
       System.err.println( str );

       String[] tiles = query.getTitle();
       assertTrue(tiles[0].equals("eml200-quick-search-query"));
       QueryTypeNamespace namespace = query.getNamespace();
       System.out.println("The namespace is "+namespace.get_value());
       assertTrue(
          (namespace.toString()).trim().equals("eml://ecoinformatics.org/eml-2.0.0"));
       String [] returnFields  = query.getReturnField();
       assertTrue(returnFields[0].equals("dataset/title"));
       assertTrue(returnFields[1].equals("entityName"));
       ANDType and = query.getAND();

       ConditionType[] conditionList = and.getOR()[0].getCondition();
       ConditionType condition = conditionList[0];
       OperatorType operator = condition.getOperator();
       String operatorStr = operator.getValue();
       assertTrue(operatorStr.equals("LIKE"));
       String expressionString = condition.getConcept();
       assertTrue(expressionString.equals("dataset/title"));
       String searchValue = condition.get_value();
       assertTrue(searchValue.equals(SEARCHVALUE));


    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail("Error getQuickSearchEcoGridQueryTest: " + e.getMessage());
    }
  }

  /**
   *  The method to test transformResultset method. The original method will
   *  transform ecogrid resultset to ResultRecord object array which will be
   *  used in kepler
   */
   public void  transformResultsetTest()
   {
     try
     {
       ResultsetType resultType = generateResultSet();
       CompositeEntity resultsRoot = new CompositeEntity();
       resultsRoot.setName("resultset");
       ResultRecord[] resultRecord = eml2Metadata.transformResultset(
           resultType, ENDPOINT, resultsRoot);
       // compare record 1
       ResultRecord record1 = resultRecord[0];
       String name1 = record1.getName();
       assertTrue(name1.equals(TITLE1));
       StringAttribute idAttr1 = (StringAttribute) record1.getAttribute(
           "recordId");
       String id1 = idAttr1.getExpression();
       assertTrue(id1.equals(ID1));
       ResultRecordDetail tetail11 = (ResultRecordDetail)
           record1.getAttribute(ENTITY11);
       assertTrue(tetail11.getName().equals(ENTITY11));
       

       
     }
     catch (Exception e)
     {
       e.printStackTrace();
       fail("Error getQuickSearchEcoGridQueryTest: " + e.getMessage());
     }

   }

   /*
    * Method to generate a resultset object
    */
   private ResultsetType generateResultSet() throws Exception
   {
      ResultsetType result = new ResultsetType();
      // set system
      result.setSystem(new URI("http://knb.ecoinformatics.org")); 
      // set resultsetId
      result.setResultsetId("eml.001");
      // set meta data
      ResultsetTypeResultsetMetadata metadata = 
                                         new ResultsetTypeResultsetMetadata();
      //get the size of metacat hashtable
      int size = 2;
      // if size is zero, it start 0
      metadata.setStartRecord(1);
      metadata.setEndRecord(size);
      metadata.setRecordCount(size);
      ResultsetTypeResultsetMetadataNamespace namespace = new ResultsetTypeResultsetMetadataNamespace();
      namespace.set_value(new URI("eml://ecoinformatics.org/eml-2.0.0"));
      metadata.setNamespace(namespace);
      
      ResultsetTypeResultsetMetadataRecordStructureReturnField[] array = 
              new ResultsetTypeResultsetMetadataRecordStructureReturnField[2];
      ResultsetTypeResultsetMetadataRecordStructureReturnField field1 = new ResultsetTypeResultsetMetadataRecordStructureReturnField();
      field1.setId("f1");
      field1.setName("dataset/title");
      ResultsetTypeResultsetMetadataRecordStructureReturnField field2 = new ResultsetTypeResultsetMetadataRecordStructureReturnField();
      field2.setId("f2");
      field1.setName("entityName");
      array[0]=field1;
      array[1]=field2;
      ResultsetTypeResultsetMetadataRecordStructure structure = new ResultsetTypeResultsetMetadataRecordStructure();
      structure.setReturnField(array);
      metadata.setRecordStructure(structure);
      result.setResultsetMetadata(metadata);
      
      // ---------handle record 
      // create a AnyRecordArray
      ResultsetTypeRecord[] recordList = new ResultsetTypeRecord[1];
      ResultsetTypeRecord record1 = new ResultsetTypeRecord();
      ResultsetTypeRecordReturnField[] array2= new ResultsetTypeRecordReturnField[2];
      ResultsetTypeRecordReturnField return1 = new ResultsetTypeRecordReturnField();
      return1.setId("f1");
      return1.set_value(TITLE1);
      ResultsetTypeRecordReturnField return2 = new ResultsetTypeRecordReturnField();
      return1.setId("f2");
      return1.set_value(ENTITY11);
      array2[0]=return1;
      array2[1]=return2;
      record1.setReturnField(array2);
      recordList[0]=record1;
     
      // set record array to ecogrid result objec
      result.setRecord(recordList);
     
     return result;
   }


}
