/**
 *  '$RCSfile: LSIDTreeTest.java,v $'
 *  Copyright: 2000 Regents of the University of California and the
 *              National Center for Ecological Analysis and Synthesis
 *    Authors: @authors@
 *    Release: @release@
 *
 *   '$Author: berkley $'
 *     '$Date: 2005/12/10 00:16:08 $'
 * '$Revision: 1.5 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package test.org.kepler.objectmanager.lsid;

import edu.ucsb.nceas.configxml.ConfigXML;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import java.util.*;
import java.io.*;

import org.kepler.objectmanager.*;
import org.kepler.objectmanager.lsid.*;

public class LSIDTreeTest  extends TestCase
{
  
  public LSIDTreeTest(String name)
  {
    super(name);
  }


  /**
   * Establish a testing framework by initializing appropriate objects
   */
  public void setUp()
  {
  }

  /**
   * Release any objects after tests are complete
   */
  public void tearDown()
  {
  }

  /**
   * Create a suite of tests to be run together
   */
  public static Test suite()
  {
    TestSuite suite = new TestSuite();
    suite.addTest(new LSIDTreeTest("initialize"));
    suite.addTest(new LSIDTreeTest("addLSIDTest"));
    suite.addTest(new LSIDTreeTest("getNextRevisionTest"));
    suite.addTest(new LSIDTreeTest("findNextRevisionTest"));
    suite.addTest(new LSIDTreeTest("getNextObjectTest"));
    suite.addTest(new LSIDTreeTest("saveAndReadTreeTest"));
    return suite;
  }

  /**
   * Run an initial test that always passes to check that the test
   * harness is working.
   */
  public void initialize()
  {
    assertTrue(1 == 1);
  }
  
  /**
   * test the addlsid method.
   */
  public void addLSIDTest()
  {
    try
    {
      LSIDTree tree = new LSIDTree();
      KeplerLSID lsid1 = new KeplerLSID("urn:lsid:kepler-project.org:actor:1:1");
      KeplerLSID lsid2 = new KeplerLSID("urn:lsid:kepler-project.org:actor:2:1");
      KeplerLSID lsid3 = new KeplerLSID("urn:lsid:kepler-project.org:actor:2:2");
      KeplerLSID lsid4 = new KeplerLSID("urn:lsid:localhost:actor:1:1");
      KeplerLSID lsid5 = new KeplerLSID("urn:lsid:localhost:actor:2:1");
      KeplerLSID lsid6 = new KeplerLSID("urn:lsid:localhost:actor:2:2");
      tree.addLSID(lsid1);
      tree.addLSID(lsid2);
      tree.addLSID(lsid3);
      tree.addLSID(lsid4);
      tree.addLSID(lsid5);
      tree.addLSID(lsid6);
      try
      {
        tree.addLSID(lsid1);
        fail("should have thrown an exception because the lsid is already present.");
      }
      catch(LSIDTreeException lsidte)
      {
        
      }
    }
    catch(Exception e)
    {
      fail("error in addLSIDTest: " + e.getMessage());
    }
  }
  
  /**
   * test the next revision method.
   */
  public void getNextRevisionTest()
  {
    try
    {
      LSIDTree tree = new LSIDTree();
      KeplerLSID lsid1 = new KeplerLSID("urn:lsid:localhost:actor:5:1");
      KeplerLSID lsid2 = new KeplerLSID("urn:lsid:localhost:actor:5:2");
      KeplerLSID lsid3 = new KeplerLSID("urn:lsid:localhost:actor:5:3");
      KeplerLSID lsid4 = new KeplerLSID("urn:lsid:localhost:actor:6:1");
      tree.addLSID(lsid1);
      tree.addLSID(lsid2);
      tree.addLSID(lsid3);
      KeplerLSID nextRevLsid = tree.getNextRevision(lsid1);
      assertEquals(new Integer(nextRevLsid.getRevision()).intValue(), 4);
      KeplerLSID newRevLsid = tree.getNextRevision(lsid4);
      assertEquals(new Integer(newRevLsid.getRevision()).intValue(), 1);
    }
    catch(Exception e)
    {
      fail("error in getNextRevisionTest: " + e.getMessage());
    }
  }
  
  /**
   * test the find next revision method.
   */
  public void findNextRevisionTest()
  {
    try
    {
      LSIDTree tree = new LSIDTree();
      KeplerLSID lsid1 = new KeplerLSID("urn:lsid:localhost:actor:5:1");
      KeplerLSID lsid2 = new KeplerLSID("urn:lsid:localhost:actor:5:2");
      KeplerLSID lsid3 = new KeplerLSID("urn:lsid:localhost:actor:5:3");
      KeplerLSID lsid4 = new KeplerLSID("urn:lsid:localhost:actor:6:1");
      tree.addLSID(lsid1);
      tree.addLSID(lsid2);
      tree.addLSID(lsid3);
      KeplerLSID nextRevLsid = tree.findNextRevision(lsid1);
      assertEquals(new Integer(nextRevLsid.getRevision()).intValue(), 4);
      tree.addLSID(nextRevLsid);
      KeplerLSID newRevLsid = tree.getNextRevision(lsid4);
      assertEquals(new Integer(newRevLsid.getRevision()).intValue(), 1);
    }
    catch(Exception e)
    {
      fail("error in findNextRevisionTest: " + e.getMessage());
    }
  }
  
  /**
   * test the next object method
   */
  public void getNextObjectTest()
  {
    try
    {
      LSIDTree tree = new LSIDTree();
      KeplerLSID lsid1 = new KeplerLSID("urn:lsid:localhost:actor:10:1");
      KeplerLSID lsid2 = new KeplerLSID("urn:lsid:localhost:actor:15:1");
      KeplerLSID lsid3 = new KeplerLSID("urn:lsid:localhost:actor:15:2");
      
      tree.addLSID(lsid1);
      KeplerLSID nextRevLsid = tree.getNextObject(lsid1);
      //this one is already in there so it should send the next obj back
      assertEquals(11, new Integer(nextRevLsid.getObject()).intValue());
      
      //this isn't already in the db, so it should send back the same lsid
      KeplerLSID newRevLsid = tree.getNextObject(lsid2);
      assertEquals(15, new Integer(newRevLsid.getObject()).intValue());

      KeplerLSID newRevLsid2 = tree.getNextObject(lsid3);
      assertEquals(16, new Integer(newRevLsid2.getObject()).intValue());
    }
    catch(Exception e)
    {
      fail("error in getNextObjectTest: " + e.getMessage());
    }
  }
  
  /**
   * test writing the tree out to disk.
   */
  public void saveAndReadTreeTest()
  {
    try
    {
      LSIDTree tree = new LSIDTree();
      KeplerLSID lsid1 = new KeplerLSID("urn:lsid:localhost:actor:10:1");
      KeplerLSID lsid2 = new KeplerLSID("urn:lsid:localhost:actor:15:1");
      KeplerLSID lsid3 = new KeplerLSID("urn:lsid:localhost:actor:15:2");
      KeplerLSID lsid4 = new KeplerLSID("urn:lsid:localhost:actor:15:3");
      KeplerLSID lsid5 = new KeplerLSID("urn:lsid:localhost:actor:15:4");
      KeplerLSID lsid6 = new KeplerLSID("urn:lsid:localhost:actor:15:5");
      KeplerLSID lsid7 = new KeplerLSID("urn:lsid:localhost:class:10:1");
      KeplerLSID lsid8 = new KeplerLSID("urn:lsid:localhost:class:10:2");
      KeplerLSID lsid9 = new KeplerLSID("urn:lsid:lsid.ecoinformatics.org:class:10:1");
      KeplerLSID lsid10 = new KeplerLSID("urn:lsid:lsid.ecoinformatics.org:class:10:2");

      tree.addLSID(lsid1);
      tree.addLSID(lsid2);
      tree.addLSID(lsid3);
      tree.addLSID(lsid4);
      tree.addLSID(lsid5);
      tree.addLSID(lsid6);
      tree.addLSID(lsid7);
      tree.addLSID(lsid8);
      tree.addLSID(lsid9);
      tree.addLSID(lsid10);

      tree.saveTree();
      
      LSIDTree tree2 = new LSIDTree();
      tree2.readTree();
      try
      {
        tree2.addLSID(lsid1);
        fail("should have thrown an exception because the lsid should " +
          "already exist.");
      }
      catch(LSIDTreeException lsidte)
      {
        //blank intentionally
      }
    }
    catch(Exception e)
    {
      fail("unexpected exception in saveTreeTest(): " + e.getMessage());
    }
    
  }
}