/**

 *    '$RCSfile: TestClient.java,v $'

 *    Copyright: 2002 Regents of the University of California and the

 *              National Center for Ecological Analysis and Synthesis

 *    Authors: @authors@

 *    Release: @release@

 *

 *   '$Author: berkley $'

 *   '$Date: 2004/04/30 20:03:06 $'

 *   '$Revision: 1.3 $'

 *

 * This program is free software; you can redistribute it and/or modify

 * it under the terms of the GNU General Public License as published by

 * the Free Software Foundation; either version 2 of the License, or

 * (at your option) any later version.

 *

 * This program is distributed in the hope that it will be useful,

 * but WITHOUT ANY WARRANTY; without even the implied warranty of

 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

 * GNU General Public License for more details.

 *

 * You should have received a copy of the GNU General Public License

 * along with this program; if not, write to the Free Software

 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

 *

 */



package org.ecoinformatics.seek.webservice.gis.grass;



import java.io.*;

import java.util.*;



import electric.registry.Registry;

import electric.registry.RegistryException;



import org.ecoinformatics.seek.webservice.gis.grass.GrassWebServiceInterface;

import org.ecoinformatics.seek.webservice.gis.grass.GrassWebServiceException;

/**

 * Class example of connecting to the seek web services.

 */

public class TestClient

{

  //the address of the web service host that you want to run these processes from

  private static String hostaddr =

    "http://localhost:8005/seek/grass.wsdl";



  /**

   * main

   */

  public static void main( String[] args ) throws Exception

  {

    if(args[0].equals("arcImport") &&

       args.length == 3)

    {

      handleArcImport(args);

    }

    else if(args[0].equals("arcExport") &&

       args.length == 2)

    {

      handleArcExport(args);

    }

    else if(args[0].equals("asciiImport") &&

       args.length == 5)

    {

      handleAsciiImport(args);

    }

    else if(args[0].equals("asciiInfo") &&

       args.length == 2)

    {

      handleAsciiInfo(args);

    }

    else if(args[0].equals("rasterInfo") &&

       args.length == 2)

    {

      handleRasterInfo(args);

    }

    else if(args[0].equals("vectorInfo") &&

       args.length == 2)

    {

      handleRasterInfo(args);

    }

    else if(args[0].equals("createHull") &&

       args.length == 3)

    {

      handleCreateHull(args);

    }

    else if(args[0].equals("createVectorSupport") &&

       args.length == 2)

    {

      handleCreateVectorSupport(args);

    }

    else if(args[0].equals("convertVectorToRaster") &&

       args.length == 3)

    {

      handleConvertVectorToRaster(args);

    }

    else if(args[0].equals("calculateMap") &&

       args.length == 3)

    {

      handleCalculateMap(args);

    }

    else

    {

      printUsage();

    }

  }



  /**

   * runs the importArcToGrass service

   */

  private static void handleArcImport(String[] args) throws Exception

  {

    String url = hostaddr;

    GrassWebServiceInterface grassInterface =

      (GrassWebServiceInterface) Registry.bind( url,

        GrassWebServiceInterface.class );

    String fileContents = getFileContents(args[1]);



    try

    {

      grassInterface.importArcToGrass(fileContents, args[2]);

      System.out.println("file successfully imported into grass object " +

        args[2]);

    }

    catch(GrassWebServiceException gwse)

    {

      System.out.println("Execution Failed: " + gwse.getMessage());

      gwse.printStackTrace();

    }

  }



  /**

   * runs the exportGrassToArcFile service

   */

  private static void handleArcExport(String[] args) throws Exception

  {

    String url = hostaddr;

    GrassWebServiceInterface grassInterface =

      (GrassWebServiceInterface) Registry.bind( url,

        GrassWebServiceInterface.class );



    try

    {

      String output = grassInterface.exportGrassToArcFile(args[1]);

      System.out.println("grass export successful.  Arc file contents: \n" +

        output);

    }

    catch(GrassWebServiceException gwse)

    {

      System.out.println("Execution Failed: " + gwse.getMessage());

      gwse.printStackTrace();

    }

  }



  /**

   * runs the importAsciiData service

   */

  private static void handleAsciiImport(String[] args) throws Exception

  {

    String site = args[1];

    String importFileName = args[2];

    String dimensions = args[3];

    String delimiter = args[4];

    String url = hostaddr;



    GrassWebServiceInterface grassInterface =

      (GrassWebServiceInterface) Registry.bind(url,

        GrassWebServiceInterface.class );

    String fileContents = getFileContents(importFileName);



    try

    {

      grassInterface.importAsciiData(site, fileContents,

        new Integer(dimensions).intValue(), delimiter);

      System.out.println("ascii file successfully imported into grass " +

        "object " + site);

    }

    catch(GrassWebServiceException gwse)

    {

      System.out.println("Execution Failed: " + gwse.getMessage());

      gwse.printStackTrace();

    }

  }



  /**

   * runs the asciiDataInfo service

   */

  private static void handleAsciiInfo(String[] args) throws Exception

  {

    String site = args[1];

    String url = hostaddr;



    GrassWebServiceInterface grassInterface =

      (GrassWebServiceInterface) Registry.bind(url,

        GrassWebServiceInterface.class );



    try

    {

      String info = grassInterface.asciiDataInfo(site);

      System.out.println("ascii info for " + site + ": " +

        info);

    }

    catch(GrassWebServiceException gwse)

    {

      System.out.println("Execution Failed: " + gwse.getMessage());

      gwse.printStackTrace();

    }

  }



  /**

   * runs the rasterDataInfo service

   */

  private static void handleRasterInfo(String[] args) throws Exception

  {

    String map = args[1];

    String url = hostaddr;



    GrassWebServiceInterface grassInterface =

      (GrassWebServiceInterface) Registry.bind(url,

        GrassWebServiceInterface.class );



    try

    {

      String info = grassInterface.rasterDataInfo(map);

      System.out.println("raster info for " + map + ": " +

        info);

    }

    catch(GrassWebServiceException gwse)

    {

      System.out.println("Execution Failed: " + gwse.getMessage());

      gwse.printStackTrace();

    }

  }



  /**

   * runs the vectorDataInfo service

   */

  private static void handleVectorInfo(String[] args) throws Exception

  {

    String vector = args[1];

    String url = hostaddr;



    GrassWebServiceInterface grassInterface =

      (GrassWebServiceInterface) Registry.bind(url,

        GrassWebServiceInterface.class );



    try

    {

      String info = grassInterface.vectorDataInfo(vector);

      System.out.println("vector info for " + vector + ": " + info);

    }

    catch(GrassWebServiceException gwse)

    {

      System.out.println("Execution Failed: " + gwse.getMessage());

      gwse.printStackTrace();

    }

  }



  /**

   * runs the createHull service

   */

  private static void handleCreateHull(String[] args) throws Exception

  {

    String sites = args[1];

    String vect = args[2];

    String url = hostaddr;



    GrassWebServiceInterface grassInterface =

      (GrassWebServiceInterface) Registry.bind(url,

        GrassWebServiceInterface.class );



    try

    {

      grassInterface.createHull(sites, vect);

      System.out.println("Hull created successfully");

    }

    catch(GrassWebServiceException gwse)

    {

      System.out.println("Execution Failed: " + gwse.getMessage());

      gwse.printStackTrace();

    }

  }



  /**

   * runs the createVectorSupportObjects service

   */

  private static void handleCreateVectorSupport(String[] args) throws Exception

  {

    String vect = args[1];

    String url = hostaddr;



    GrassWebServiceInterface grassInterface =

      (GrassWebServiceInterface) Registry.bind(url,

        GrassWebServiceInterface.class );



    try

    {

      grassInterface.createVectorSupportObjects(vect);

      System.out.println("vector support created successfully.");

    }

    catch(GrassWebServiceException gwse)

    {

      System.out.println("Execution Failed: " + gwse.getMessage());

      gwse.printStackTrace();

    }

  }



  /**

   * runs the convertVectorToRaster service

   */

  private static void handleConvertVectorToRaster(String[] args) throws Exception

  {

    String vect = args[1];

    String rast = args[2];

    String url = hostaddr;



    GrassWebServiceInterface grassInterface =

      (GrassWebServiceInterface) Registry.bind(url,

        GrassWebServiceInterface.class );



    try

    {

      grassInterface.convertVectorToRaster(vect, rast);

      System.out.println("Raster object created from vector.");

    }

    catch(GrassWebServiceException gwse)

    {

      System.out.println("Execution Failed: " + gwse.getMessage());

      gwse.printStackTrace();

    }

  }



  /**

   * runs the calculateMap service

   */

  private static void handleCalculateMap(String[] args)  throws Exception

  {

    String rast1 = args[1];

    String rast2 = args[2];

    String url = hostaddr;



    GrassWebServiceInterface grassInterface =

      (GrassWebServiceInterface) Registry.bind(url,

        GrassWebServiceInterface.class );



    try

    {

      grassInterface.calculateMap(rast1, rast2);

      System.out.println("Map calculated.");

    }

    catch(GrassWebServiceException gwse)

    {

      System.out.println("Execution Failed: " + gwse.getMessage());

      gwse.printStackTrace();

    }

  }



  /**

   * prints the usage guideline

   */

  private static void printUsage()

  {

    System.out.println("USAGE: java TestClient <service> [params] ");

    System.out.println("services can be: arcImport <importFileName> " +

      "<grassObjectName>");

    System.out.println("                 arcExport <grassObjectName>");

    System.out.println("                 asciiImport <grassObjectName> " +

      "<importFileName> <dimensions> <delimiter ([space | tab | character])");

    System.out.println("                 asciiInfo <grassObjectName>");

    System.out.println("                 rasterInfo <grassObjectName>");

    System.out.println("                 vectorInfo <grassObjectName>");

    System.out.println("                 createHull <grassObjectName> " +

      "<outputVectorObject>");

    System.out.println("                 createVectorSupport <grassObjectName>");

    System.out.println("                 convertVectorToRaster " +

      "<grassObjectName> <outputRasterObject>" );

    System.out.println("                 calculateMap <raster1> <raster2>" );

  }



   /**

   * reads the contents of a file into a string and returns it

   */

  private static String getFileContents(String filename)

  {

    File f = new File(filename);

    try

    {

      StringBuffer sb = new StringBuffer();

      FileReader fr = new FileReader(f);

      char[] buff = new char[1024];

      int numread = fr.read(buff, 0, 1024);

      while(numread != -1)

      {

        sb.append(buff, 0, numread);

        numread = fr.read(buff, 0, 1024);

      }



      return sb.toString();

    }

    catch(Exception e)

    {

      System.out.println("Error getting the contents of file " + f.toString());

      e.printStackTrace();

      return null;

    }

  }



}

